#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ihook.PocketWidgets
{
	/// <summary>
	/// Summary description for AppDefaults.
	/// </summary>
	public class AppDefaults
	{
		public static Color DisabledBackground
		{
			get { return SystemColors.Control; }
		}
		public static Color EnabledBackground
		{
			get { return SystemColors.Window; }
		}
		static Brush enabledBrush, disabledBrush;
		public static Brush DisabledBackgroundBrush
		{
			get
			{ 
				if(disabledBrush == null)
					disabledBrush = new SolidBrush(DisabledBackground);
				return disabledBrush; 
			}
		}
		public static Brush EnabledBackgroundBrush
		{
			get
			{ 
				if(enabledBrush == null)
					enabledBrush = new SolidBrush(EnabledBackground);
				return enabledBrush; 
			}
		}

		static Brush darkBrush;
		public static Brush DarkBrush
		{
			get
			{ 
				if(darkBrush == null)
					darkBrush = new SolidBrush(Color.DarkGray);
				return darkBrush; 
			}
		}

		static Brush redBrush;
		public static Brush RedBrush
		{
			get
			{ 
				if(redBrush == null)
					redBrush = new SolidBrush(Color.Red);
				return redBrush; 
			}
		}

		static Form hiddenOne;
		internal static Form HiddenForm
		{
			get 
			{
				if(hiddenOne == null)
					hiddenOne = new Form();
				return hiddenOne;
			}
		}

		static Font font;
		public static Font Font
		{
			get 
			{
				if(font == null)
					font = HiddenForm.Font;
				return font;
			}
		}
		static Font boldfont;
		public static Font BoldFont
		{
			get 
			{
				if(boldfont == null) 
				{
					Font f = Font;
					boldfont = new Font(f.Name, f.Size, FontStyle.Bold);
				}
				return boldfont;
			}
		}

		static Pen borderpen;
		public static Pen BorderPen
		{
			get 
			{
				if(borderpen == null)
					borderpen = new Pen(SystemColors.WindowFrame);
				return borderpen;
			}
		}

		static Pen textpen;
		public static Pen TextPen
		{
			get 
			{
				if(textpen == null)
					textpen = new Pen(SystemColors.WindowText);
				return textpen;
			}
		}

		static Pen lightpen;
		public static Pen LightPen
		{
			get 
			{
				if(lightpen == null)
					lightpen = new Pen(Color.DarkGray);
				return lightpen;
			}
		}

		static Brush textbrush;
		public static Brush TextBrush
		{
			get 
			{
				if(textbrush == null)
					textbrush = new SolidBrush(SystemColors.WindowText);
				return textbrush;
			}
		}

		static Brush bluebrush;
		public static Brush BlueBrush
		{
			get 
			{
				if(bluebrush == null)
					bluebrush = new SolidBrush(SystemColors.ActiveCaption);
				return bluebrush;
			}
		}

		/// <summary>
		/// a brush for empty part of a control, light gray color
		/// </summary>
		public static Brush EmptyBrush
		{
			get
			{
				if(emptyBrush == null)
					emptyBrush = new SolidBrush(Color.DarkGray);
				return emptyBrush;
			}
		}
		static Brush emptyBrush;

		/// <summary>
		/// a brush for selected background
		/// </summary>
		public static Brush SelectionBrush
		{
			get
			{
				if(selectionBrush == null)
					selectionBrush = new SolidBrush(Color.Cyan);
				return selectionBrush;
			}
		}
		static Brush selectionBrush;

		static Brush lightbrush;
		public static Brush LightBrush
		{
			get 
			{
				if(lightbrush == null)
					lightbrush = new SolidBrush(Color.LightGray);
				return lightbrush;
			}
		}

		static Brush disabledbrush;
		public static Brush DisabledBrush
		{
			get 
			{
				if(disabledbrush == null)
					disabledbrush = new SolidBrush(SystemColors.GrayText);
				return disabledbrush;
			}
		}

		static Brush whitebrush;
		public static Brush WhiteBrush
		{
			get 
			{
				if(whitebrush == null)
					whitebrush = new SolidBrush(SystemColors.Window);
				return whitebrush;
			}
		}

		static Brush highlightBrush;
		public static Brush HighlightBrush
		{
			get 
			{
				if(highlightBrush == null)
					highlightBrush = new SolidBrush(SystemColors.Highlight);
				return highlightBrush;
			}
		}

		static Pen highlightPen;
		public static Pen HighlightPen
		{
			get 
			{
				if(highlightPen == null)
					highlightPen = new Pen(SystemColors.Highlight);
				return highlightPen;
			}
		}

		static Brush highlightTextBrush;
		public static Brush HighlightTextBrush
		{
			get 
			{
				if(highlightTextBrush == null)
					highlightTextBrush = new SolidBrush(SystemColors.HighlightText);
				return highlightTextBrush;
			}
		}
	}
}
