#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ihook.PocketWidgets
{
	/// <summary>
	/// a control which would scroll and display its Text
	/// </summary>
	public class Credit : DoubleBufferedControl
	{
		Timer refresher;
		public Credit()
		{
			refresher = new Timer();
			refresher.Tick += new EventHandler(HRefresh);
			refresher.Interval = 10;
		}
		public int Interval
		{
			get { return refresher.Interval; }
			set { refresher.Interval = value; }
		}
		int yjump = 1;
		public int YJump
		{
			get { return yjump; }
			set
			{
				if(value < 1)
					value = 1;
				yjump = value;
			}
		}
		void HRefresh(object sender, EventArgs e)
		{
			Invalidate();
			offset -= yjump;
			refresher.Enabled = false; // stop it
		}
		protected override void DoubleBufferedPaint(PaintEventArgs e)
		{
			base.DoubleBufferedPaint (e);
			if(text.Length > 0)
				DrawText(e);
			GC.Collect();
			refresher.Enabled = true; // restart it
		}
		string[] text = new string[0];
		public override string Text
		{
			set
			{
				base.Text = value;
				if(value == null || value == "")
					text = new string[0];
				else
					text = value.Split('\n');
			}
		}
		int offset;
		protected void DrawText(PaintEventArgs e)
		{
			int H = (int) e.Graphics.MeasureString("apqt", Font).Height;
			Size size = Size;
			int x = 0, y0 = offset;
			while(y0 < size.Height)
			{
				foreach(string str in text)
				{
					if(y0 + H > 0)
					{
						SizeF s = e.Graphics.MeasureString(str, Font);
						x = (size.Width - (int) s.Width) / 2;
						e.Graphics.DrawString(str, Font, AppDefaults.TextBrush, x, y0);
					}
					y0 += H;
				}
				int dy = y0 - offset;
				if(y0 < 0)
					offset += dy;
			}
		}
	}
}
