#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using System.Drawing.Imaging;

//
// This code is gracefull adapted from an article on
// http://www.opennetcf.org
//
namespace ihook.PocketWidgets
{
	public delegate void ListBoxDataHandler(object item, out string text, out Icon icon);
	public delegate object ListBoxDataValue(object item);

	/// <summary>
	/// Summary description for CustomListBox.
	/// </summary>
	public class CustomListBox : OwnerDrawnList
	{
		const int DRAW_OFFSET  = 4;
		/*private bool wrapText = false;

		public bool WrapText
		{
			get
			{
				return wrapText;
			}
			set
			{
				wrapText = value;
			}
		}*/
		
		ListBoxDataHandler dataHandler;
		public ListBoxDataHandler DataHandler
		{
			get 
			{
				if(dataHandler == null)
					dataHandler = new ListBoxDataHandler(DefaultDataHandler);
				return dataHandler;
			}
			set { dataHandler = value; }
		}

		ListBoxDataValue dataValue;
		public ListBoxDataValue DataValue
		{
			get 
			{
				if(dataValue == null)
					dataValue = new ListBoxDataValue(DefaultDataValue);
				return dataValue;
			}
			set { dataValue = value; }
		}

		object DefaultDataValue(object item) { return item; }
		void DefaultDataHandler(object item, out string text, out Icon icon)
		{
			text = "";
			icon = null;
			if(item != null)
				text = item.ToString();
		}

		public CustomListBox()
		{
			this.ShowScrollbar = true;
			this.ForeColor = Color.Black;
			//Set the item's height
			Graphics g = this.CreateGraphics();
//			if (wrapText)
//			  this.ItemHeight = 2 * Math.Max((int)(g.MeasureString("A", this.Font).Height), this.ItemHeight) + 2;
//			else
			  this.ItemHeight = Math.Max((int)(g.MeasureString("A", this.Font).Height), this.ItemHeight) + 4;

			g.Dispose();
		}

		public override object SelectedItem 
		{
			get { return DataValue(base.SelectedItem); }
			set
			{
				if(value == null || Items == null)
					SelectedIndex = -1;
				else
				{
					IList l = Items;
					for(int i=0; i<l.Count; i++)
						if(value == DataValue(l[i]))
						{
							SelectedIndex = i;
							return;
						}
				}
			}
		}
		protected override void OnDrawItem(object sender, DrawItemEventArgs e)
		{
			Brush textBrush; //Brush for the text
   
			Rectangle rc = e.Bounds;
			rc.X += DRAW_OFFSET;
			//Check if the item is selected
			if (e.State == DrawItemState.Selected)
			{
				//Highlighted
				e.DrawBackground();
				textBrush = AppDefaults.HighlightTextBrush;
			}
			else
			{
				//Change the background for every even item
//				if ((e.Index % 2) == 0)
//				{
//					e.DrawBackground(AppDefaults.LightBrush);
//				}
//				else
				{
					e.DrawBackground(AppDefaults.WhiteBrush);
				}
								
				textBrush = new SolidBrush(this.ForeColor);
			}
			
			//Get the ListItem
			object item = this.Items[e.Index];
			Icon icon;
			string text;
			DataHandler(item, out text, out icon);

//			//Check if the item has a image
			if (icon != null)
			{
				e.Graphics.DrawIcon(icon, 2, rc.Y+1);
				rc.X+= icon.Width + 2;
			}
			
			//Draw item's text
			e.Graphics.DrawString(text, e.Font, textBrush,  rc.X, rc.Y);
			//Draw the line
			//e.Graphics.DrawLine(new Pen(Color.Navy), 0, e.Bounds.Bottom, e.Bounds.Width, e.Bounds.Bottom);
			//Call the base's OnDrawEvent	
			base.OnDrawItem (sender, e);
		}
		protected override int LineWidth(Graphics g, int line)
		{
			object item = this.Items[line];
			Icon icon;
			string text;
			DataHandler(item, out text, out icon);

			int ret = 0;
			if(icon != null)
				ret += icon.Width + 2;
			ret += (int) g.MeasureString(text, Font).Width;
			ret += 7;
			return ret;
		}

		private Color BackgroundImageColor(Image image)
		{
			Bitmap bmp = new Bitmap(image);
			Color ret = bmp.GetPixel(0, 0);
			return ret;
		}
	}
	//ListItem class
	/*public class ListItem
	{
		private string text = "";
		private int imageIndex = -1;

		public string Text
		{
			get
			{
				return text;
			}
			set
			{
				text = value;
			}
		}

		public int ImageIndex
		{
			get
			{
				return imageIndex;
			}
			set
			{
				imageIndex = value;
			}
		}
	}*/
}
