#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Icons;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// Summary description for BoolEditor.
	/// </summary>
	public class BoolEditor : Control, IEditor
	{
		public BoolEditor()
		{
			Size = new Size(40, 20);
		}

		string trueText, falseText;
		public string TrueText
		{
			get { return trueText; }
			set 
			{
				trueText = value;
				Invalidate();
			}
		}
		public string FalseText
		{
			get { return falseText; }
			set 
			{
				falseText = value;
				Invalidate();
			}
		}

		Icon icoCheck = IconManager.GetIcon("checked.ico");
		Icon icoUncheck = IconManager.GetIcon("unchecked.ico");
		protected override void OnPaint(PaintEventArgs e)
		{
			if(isNull)
				return;
			Font f = Font;
			int yIcon = (Height - 16) / 2;
			int yText = (int)((Height - f.Size) / 2);
			e.Graphics.DrawIcon(Value ? icoCheck : icoUncheck, 1, yIcon);
			string s = Value ? TrueText : FalseText;
			if(s == null)
				s = "";
			e.Graphics.DrawString(s, f, AppDefaults.TextBrush, 18, yText);
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			if(Enabled)
				Value = ! Value;
			else
				WinAPI.Beep();
		}


		#region IEditor Members

		public Control Control { get { return this; } }

		bool bValue;
		public bool Value
		{
			get { return bValue; }
			set 
			{ 
				isNull = false;
				bValue = value; 
				Invalidate();
				if(ValueChanged != null)
					ValueChanged(this, EventArgs.Empty);
			}
		}
		object IEditor.Value
		{
			get 
			{ 
				if(isNull)
					return null;
				return Value; 
			}
			set
			{
				if(value == DBNull.Value || value == null)
					IsNull = true;
				else
					Value = (bool) value;
			}
		}
		bool isNull;
		public bool IsNull
		{
			get { return isNull; }
			set
			{
				if(isNull == value)
					return;
				isNull = value;
				Invalidate();
				if(ValueChanged != null)
					ValueChanged(this, EventArgs.Empty);
			}
		}
		public event System.EventHandler ValueChanged;

		#endregion
	}
}
