#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Icons;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// Summary description for DateTimeEditor.
	/// </summary>
	public class DateTimeEditor : Control, IEditor
	{
		TimeEditor timeEdit = new TimeEditor();
		DateEditor dateEdit = new DateEditor();

		public DateTimeEditor()
		{
			Size s1 = timeEdit.Size;
			Size s2 = dateEdit.Size;
			Size = new Size((s1.Width+s2.Width)/2, (s1.Height+s2.Height)/2);

			Controls.Add(timeEdit.Control);
			Controls.Add(dateEdit.Control);
			timeEdit.ValueChanged += new EventHandler(timeEdit_ValueChanged);
			dateEdit.ValueChanged += new EventHandler(dateEdit_ValueChanged);
		}
		protected override void OnParentChanged(EventArgs e)
		{
			base.OnParentChanged (e);
			dateEdit.Popup.PopDown();
		}
		protected override void OnResize(EventArgs e)
		{
			base.OnResize (e);
			int w = Width;
			int h = Height;
			int w1 = w-w/3; // prevent rounding pixel loss
			int w2 = w/3;
			timeEdit.Control.Size = new Size(w1+1, h);
			dateEdit.Control.Size = new Size(w2, h);
			dateEdit.Control.Location = new Point(w1, 0);
		}
		bool dontUpdate = false;
		private void timeEdit_ValueChanged(object sender, EventArgs e)
		{
			SetDateTime(dtValue, timeEdit.Value);
		}
		private void dateEdit_ValueChanged(object sender, EventArgs e)
		{
			SetDateTime(dateEdit.Value, dtValue);
		}
		void SetDateTime(DateTime date, DateTime time)
		{
			if(dontUpdate)
				return;
			dtValue = new DateTime(date.Year, date.Month, date.Day, time.Hour, time.Minute, time.Second, 0);
			if(ValueChanged != null)
				ValueChanged(this, EventArgs.Empty);
		}
		protected override void OnEnabledChanged(EventArgs e)
		{
			base.OnEnabledChanged(e);
			timeEdit.Enabled = this.Enabled;
			dateEdit.Enabled = this.Enabled;
		}

		#region IEditor Members

		public Control Control
		{
			get { return this; }
		}
		object IEditor.Value
		{
			get 
			{ 
				if(IsNull)
					return null;
				return Value; 
			}
			set 
			{ 
				if(value == null || value.Equals(DBNull.Value))
					IsNull = true;
				else
					Value = (DateTime) value; 
			}
		}
		public bool IsNull
		{
			get { return timeEdit.IsNull; }
			set
			{
				dontUpdate = true;
				try
				{
					timeEdit.IsNull = value;
					dateEdit.IsNull = value;
				}
				finally { dontUpdate = false; }
				SetDateTime(dateEdit.Value, timeEdit.Value);
			}
		}
		DateTime dtValue;
		public DateTime Value
		{
			get { return dtValue; }
			set
			{
				dontUpdate = true;
				try
				{
					timeEdit.Value = value;
					dateEdit.Value = value;
				}
				finally { dontUpdate = false; }
				SetDateTime(value, value);
			}
		}
		public event System.EventHandler ValueChanged;

		#endregion
	}
}
