#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Icons;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// a sort of editable ComboBox ....
	/// </summary>
	public class EditableListEditor : Control, IEditor
	{
		IEditor mainEditor;
		CustomListBox list;
		PopupPanel popup;
		bool initializing = false;

		public EditableListEditor(IEditor cellEditor) : this()
		{
			CellEditor = cellEditor;
		}
		public EditableListEditor()
		{
			Size = new Size(100, 21);
			popup = new PopupPanel(ContentAlignment.TopRight);
			list = new CustomListBox();
			list.SelectedIndexChanged += new EventHandler(HSelectedIndexChanged);
			list.Parent = popup;
			popup.HScrollEnabled = false;
			popup.VScrollEnabled = false;
		}
		protected override void OnParentChanged(EventArgs e)
		{
			base.OnParentChanged (e);
			popup.PopDown();
		}

		public CustomListBox ListBox { get { return list; }  }
		public IList ListSource
		{
			get { return ListBox.Items; }
			set 
			{ 
				popup.PopDown();
				if(value == ListBox.Items)
					return;
				ListBox.Items = value; 
				object sel = Value;
				if(value != null && value.Contains(sel))
					return;
				object sel2 = value==null||value.Count==0 ? null : value[0];
				if(sel2 == sel) // null value
					return;
				Value = ListBox.SelectedItem;
			}
		}
		public void SetValues(object selected, IList list)
		{
			popup.PopDown();
			ListBox.Items = list; 
			Value = selected;
		}
		void ResizeEditor()
		{
			Size s = Size;
			if(s.Width > 0 && s.Height > 0 && mainEditor != null)
			{
				Rectangle r = ClientRectangle;
				r.Width -= 14;
				//r.Inflate(-1, -1);
				mainEditor.Control.Bounds = r;
			}
		}
		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);
			Pen pen = Enabled ? AppDefaults.TextPen : AppDefaults.LightPen;
			Brush b = Enabled ? AppDefaults.TextBrush : AppDefaults.DarkBrush;
			int w = Width, h = Height;
			Point[] arrow = new Point[3];
			arrow[0] = new Point(w-12, h/2-1);
			arrow[1] = new Point(w-3, h/2-1);
			arrow[2] = new Point(w-8, h/2+3);
			e.Graphics.DrawRectangle(pen, 0, 0, w-1, h-1);
			e.Graphics.DrawLine(pen, w-15, 0, w-15, h);
			e.Graphics.DrawLine(pen, w-11, h/2-1, w-5, h/2-1);
			e.Graphics.DrawLine(pen, w-10, h/2, w-6, h/2);
			e.Graphics.DrawLine(pen, w-9, h/2+1, w-7, h/2+1);
			e.Graphics.DrawLine(pen, w-8, h/2+3, w-8, h/2);
			e.Graphics.FillPolygon(b, arrow);
		}
		public IEditor CellEditor
		{
			set
			{
				if(mainEditor != null)
				{
					mainEditor.ValueChanged -= new EventHandler(HValueChanged);
					mainEditor.Control.Parent = null;
				}
				mainEditor = value;
				if(mainEditor != null)
				{
					mainEditor.Control.Parent = this;
					ResizeEditor();
					mainEditor.ValueChanged += new EventHandler(HValueChanged);
					mainEditor.Enabled = Enabled;
				}
			}
			get { return mainEditor; }
		}
		private void HSelectedIndexChanged(object sender, EventArgs e)
		{
			if(initializing)
				return;
			bool b = popup.Popping;
			popup.PopDown();
			if(b && mainEditor != null)
				mainEditor.Value = list.SelectedItem;
		}
		private void HValueChanged(object sender, EventArgs e)
		{
			if(ValueChanged != null)
				ValueChanged(this, e);
		}
		protected override void OnResize(EventArgs e)
		{
			base.OnResize (e);
			ResizeEditor();
		}
		protected override void OnEnabledChanged(EventArgs e)
		{
			base.OnEnabledChanged (e);
			if(mainEditor != null)
				mainEditor.Enabled = Enabled;
		}
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			if(!Enabled)
				return;
			if(popup.Popping)
				popup.PopDown();
			else
				ShowPopup();
		}
		protected void ShowPopup()
		{
			initializing = true;
			try
			{
				ResizePopup();
				popup.PopUp(this);
				if(mainEditor != null)
					list.SelectedItem = mainEditor.Value;
			}
			finally { initializing = false; }
		}
		void ResizePopup()
		{
			Size s = list.DesiredSize(5);
			int max = TopLevelControl.Width / 2 - 5;
			if(s.Width > max)
				s.Width = max;
			if(s.Width < Width)
				s.Width = Width;
			list.Size = s;
			popup.Size = list.Size;
		}

		#region IEditor Members

		public Control Control
		{
			get { return this; }
		}

		public object Value
		{
			get 
			{
				if(mainEditor == null)
					return null;
				return mainEditor.Value;
			}
			set
			{
				if(mainEditor != null)
					mainEditor.Value = value;
			}
		}
		public bool IsNull
		{
			get 
			{
				if(mainEditor == null)
					return true;
				return mainEditor.IsNull;
			}
			set
			{
				if(mainEditor != null)
					mainEditor.IsNull = value;
			}
		}

		public event System.EventHandler ValueChanged;

		#endregion
	}
}
