#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Icons;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// a class to mix with ListEditor to emulate ComboBox.
	/// </summary>
	public class LabelEditor : Control, IEditor
	{
		public LabelEditor() : this(true)
		{
		}
		public LabelEditor(bool grayColor) 
		{
			if(grayColor)
				ForeColor = Color.DarkGray;
			Size = new Size(50, 20);
		}

		public override Color ForeColor
		{
			set
			{
				if(textBrush != null)
					textBrush.Dispose();
				textBrush = null;
				base.ForeColor = value;
				Invalidate();
			}
		}
		protected override void Dispose(bool disposing)
		{
			base.Dispose (disposing);
			if(disposing)
			{
				if(textBrush != null)
					textBrush.Dispose();
				textBrush = null;
			}
		}
		SolidBrush textBrush;
		protected Brush TextBrush
		{
			get 
			{
				if(textBrush == null)
					textBrush = new SolidBrush(ForeColor);
				return textBrush;
			}
		}
		public override string Text
		{
			set
			{
				base.Text = value;
				Invalidate();
			}
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);
			string s = Text;
			if(s == null)
				s = "";
			e.Graphics.DrawString(s, Font, TextBrush, 2, 2);
			e.Graphics.DrawRectangle(AppDefaults.TextPen, 0, 0, Width-1, Height-1);
		}

		#region IEditor Members

		public Control Control
		{
			get { return this; }
		}

		object objectValue;
		public virtual object Value
		{
			get { return objectValue; }
			set 
			{ 
				if(value == null || value.Equals(DBNull.Value))
				{
					objectValue = null;
					Text = null;
				}
				else
				{
					objectValue = value;
					Text = value.ToString(); 
				}

				if(ValueChanged != null)
					ValueChanged(this, EventArgs.Empty);
			}
		}

		public virtual bool IsNull
		{
			get { return objectValue == null; }
			set 
			{
				if(value)
					Value = null;
			}
		}

		public event System.EventHandler ValueChanged;

		#endregion
	}
}
