#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Icons;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// Summary description for TimeEditor.
	/// </summary>
	public class TimeEditor : DoubleBufferedControl, IEditor
	{
		public TimeEditor()
		{
			SizeF sf = BackBuffer.Graphics.MeasureString("123:456:789", Font);
			Size s = sf.ToSize();
			s.Width += 4;
			s.Height += 4;
			Size = s;
		}
		protected virtual void Fire()
		{
			if(ValueChanged != null)
				ValueChanged(this, EventArgs.Empty);
		}
		#region IEditor Members

		public Control Control
		{
			get { return this; }
		}

		bool isNull;
		public bool IsNull
		{
			get { return isNull; }
			set
			{
				if(isNull == value)
					return;
				isNull = value;
				selected = -1;
				Invalidate();
				Fire();
			}
		}
		object IEditor.Value
		{
			get 
			{ 
				if(isNull)
					return null;
				return Value; 
			}
			set 
			{ 
				if(value == null || value == DBNull.Value)
					IsNull = true;
				else
					Value = (DateTime) value; 
			}
		}
		DateTime val = DateTime.Now;
		public DateTime Value
		{
			set 
			{ 
				isNull = false;
				val = value; 
				selected = -1;
				Invalidate();
				Fire();
			}
			get { return val; }
		}

		public event System.EventHandler ValueChanged;

		#endregion

		bool showSec = true;
		public bool ShowSeconds
		{
			get { return showSec; }
			set
			{
				showSec = value;
				Invalidate();
			}
		}
		string ValToS(int n)
		{
			char[] c = new char[2];
			if(n < 9)
			{
				c[0] = '0';
				c[1] = (char)('0' + n);
			}
			else
			{
				c[0] = (char)('0' + (n / 10));
				c[1] = (char)('0' + (n % 10));
			}
			return new string(c);
		}
		#region -- override --
		protected override void OnLostFocus(EventArgs e)
		{
			base.OnLostFocus (e);
			selected = -1;
			Invalidate();
		}

		protected override void OnEnabledChanged(EventArgs e)
		{
			selected = -1;
			base.OnEnabledChanged (e);
		}

		int selected = -1;
		protected override void DoubleBufferedPaint(PaintEventArgs e)
		{
			e.Graphics.DrawRectangle(AppDefaults.TextPen, 0, 0, Width-1, Height-1);
			if(isNull)
				return;

			int dcolumn = (int) e.Graphics.MeasureString(":", Font).Width;
			string sh = ValToS(val.Hour);
			string sm = ValToS(val.Minute);
			string ss = ValToS(val.Second);
			int dh = (int) e.Graphics.MeasureString(sh, Font).Width;
			int dm = (int) e.Graphics.MeasureString(sm, Font).Width;
			int ds = (int) e.Graphics.MeasureString(ss, Font).Width;

			int dstr = dh+dcolumn+dm;
			if(showSec)
				dstr += dcolumn+ds;

			Brush brush = Enabled ? AppDefaults.TextBrush : AppDefaults.EmptyBrush;
			int x = Width - dstr - 2;
			e.Graphics.DrawString(
				sh,
				Font,
				selected==0 ? AppDefaults.RedBrush : brush,
				x, 2);
			x += dh;
			e.Graphics.DrawString(
				":",
				Font,
				brush,
				x, 2);
			x += dcolumn;
			e.Graphics.DrawString(
				sm,
				Font,
				selected==1 ? AppDefaults.RedBrush : brush,
				x, 2);
			x += dm;
			if(showSec)
			{
				e.Graphics.DrawString(
					":",
					Font,
					brush,
					x, 2);
				x += dcolumn;
				e.Graphics.DrawString(
					ss,
					Font,
					selected==2 ? AppDefaults.RedBrush : brush,
					x, 2);
			}
		}
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			if(!Enabled)
			{
				WinAPI.Beep(WinAPI.BeepSound.Asterisk);
				return;
			}
			if(isNull)
			{
				isNull = false;
				Refresh();
			}
			if(Visible && TopLevelControl is Form)
			{
				if(!Focused)
				{
					Focus();
					return;
				}
			}

			Graphics g = BackBuffer.Graphics;
			int dcolumn = (int) g.MeasureString(":", Font).Width;
			string sh = ValToS(val.Hour);
			string sm = ValToS(val.Minute);
			string ss = ValToS(val.Second);
			int dh = (int) g.MeasureString(sh, Font).Width;
			int dm = (int) g.MeasureString(sm, Font).Width;
			int ds = (int) g.MeasureString(ss, Font).Width;

			int dstr = dh+dcolumn+dm;
			if(showSec)
				dstr += dcolumn+ds;
			
			int xh = Width - dstr - 2;
			int xm = xh + dh + dcolumn;
			int xs = xm + dm + dcolumn;
			
			int news = -1;
			if(e.X > xh && e.X < xh+dh)
				news = 0;
			else if(e.X > xm && e.X < xm+dm)
				news = 1;
			else if(e.X > xs && e.X < xs+ds)
				news = 2;

			if(news != selected)
			{
				selected = news;
				Invalidate();
			}
		}
		protected override void OnKeyPress(KeyPressEventArgs e)
		{
			base.OnKeyPress (e);
			if(Enabled && e.KeyChar >= '0' && e.KeyChar <= '9')
			{
				int n = e.KeyChar - '0';
				int Y = val.Year;
				int M = val.Month;
				int D = val.Day;
				int h = val.Hour;
				int m = val.Minute;
				int s = val.Second;
				int c;
				switch(selected)
				{
					default:
						return;
					case 0:
						c = h%10;
						if(c>2)
							c = 0;
						h = c * 10 + n;
						if(h>23)
							h = n;
						break;
					case 1:
						c = m%10;
						if(c>5)
							c = 0;
						m = c * 10 + n;
						break;
					case 2:
						c = s%10;
						if(c > 5)
							c = 0;
						s = c * 10 + n;
						break;
				}
				val = new DateTime(Y, M, D, h, m, s);
				Invalidate();
				Fire();
			}
			else
				WinAPI.Beep(WinAPI.BeepSound.Asterisk);
		}
		#endregion
	}
}
