#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Windows.Forms;
using System.ComponentModel;

using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Editors
{
	/// <summary>
	/// Summary description for UpDownNumEditor.
	/// </summary>
	public class UpDownNumEditor : IEditor
	{
		NumericUpDown editor;

		public UpDownNumEditor(NumType nt) : this()
		{
			NumType = nt;
		}
		public UpDownNumEditor()
		{
			editor = new NumericUpDown();
			editor.ValueChanged += new EventHandler(HEditorValChanged);

			NumType = NumType.Int;
		}
		private void HEditorValChanged(object sender, EventArgs e)
		{
			if(ValueChanged != null && Enabled)
				ValueChanged(this, EventArgs.Empty);
		}
		public NumType NumType
		{
			get { return numType; }
			set
			{
				numType = value;
				switch(numType)
				{
					case NumType.Single:
						editor.Minimum = new decimal(float.MinValue);
						editor.Maximum = new decimal(float.MaxValue);
						break;
					case NumType.Double:
					case NumType.Decimal:
						editor.Minimum = new decimal(double.MinValue);
						editor.Maximum = new decimal(double.MaxValue);
						break;
					case NumType.Byte:
						editor.Minimum = new decimal(0);
						editor.Maximum = new decimal(byte.MaxValue);
						break;
					case NumType.SByte:
						editor.Minimum = new decimal(sbyte.MinValue);
						editor.Maximum = new decimal(sbyte.MaxValue);
						break;
					case NumType.Short:
						editor.Minimum = new decimal(short.MinValue);
						editor.Maximum = new decimal(short.MaxValue);
						break;
					case NumType.UShort:
						editor.Minimum = new decimal(0);
						editor.Maximum = new decimal(ushort.MaxValue);
						break;
					case NumType.Int:
					case NumType.Long:
						editor.Minimum = new decimal(int.MinValue);
						editor.Maximum = new decimal(int.MaxValue);
						break;
					case NumType.UInt:
					case NumType.ULong:
						editor.Minimum = new decimal(0);
						editor.Maximum = new decimal(uint.MaxValue);
						break;
				}
			}
		}
		NumType numType = NumType.Int;

		#region IEditor Members

		Control IEditor.Control
		{
			get { return editor; }
		}
		public NumericUpDown Control
		{
			get { return editor; }
		}

		public bool Enabled
		{
			get { return editor.Enabled; }
			set { editor.Enabled = value; }
		}

		object IEditor.Value
		{
			get 
			{ 
				if(editor.Text == "")
					return null;
				switch(NumType)
				{
					case NumType.Double:
						return Convert.ToDouble(Value); 
					case NumType.Single:
						return Convert.ToSingle(Value); 
					case NumType.Decimal:
						return (decimal) Value; 
					case NumType.Byte:
						return Convert.ToByte(Value); 
					case NumType.SByte:
						return Convert.ToSByte(Value); 
					case NumType.Short:
						return Convert.ToInt16(Value); 
					case NumType.UShort:
						return Convert.ToUInt16(Value); 
					case NumType.Int:
						return Convert.ToInt32(Value); 
					case NumType.UInt:
						return Convert.ToUInt32(Value); 
					case NumType.Long:
						return Convert.ToInt64(Value); 
					case NumType.ULong:
						return Convert.ToUInt64(Value); 
					default:
						return Value;
				}
			}
			set 
			{ 
				if(value == null || value == DBNull.Value)
					editor.Text = "";
				else
					Value = Convert.ToDecimal(value); 
			}
		}
		public decimal Value
		{
			get { return editor.Value; }
			set { editor.Value = value; }
		}
		public bool IsNull
		{
			get { return editor.Text == ""; }
			set 
			{
				if(value)
					editor.Text = "";
				else
					if(editor.Text == "")
						Value = decimal.Zero;
			}
		}

		public event System.EventHandler ValueChanged;

		#endregion
	}
}
