#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Windows.Forms;

namespace ihook.PocketWidgets
{
	/// <summary>
	/// this class is an event remover, it removes similar event off the 
	/// event queue if you're flooded with lenghty event like continuous 
	/// scrolling (repaint might be slow).
	/// simply call <c>myEventRemover.DoEvents()</c> and continue the
	/// execution if it return true
	/// </summary>
	public class EventRemover
	{
		bool InCall = false, InSubCall = false;

		/// <summary>
		/// This method basically call <c>Application.DoEvents()</c>
		/// but it also check if it was recursively called.
		/// If it was recursively called it immediately return false
		/// and the parent will call <c>Application.DoEvents()</c> one 
		/// more time.
		/// </summary>
		/// <returns>return true in the top level function,
		/// false if it was recursively called</returns>
		public bool DoEvents()
		{
			if(InCall)
			{
				InSubCall = true;
				return false;
			}

			InCall = true;
			try
			{
				do
				{
					InSubCall = false;
					Application.DoEvents();
				}
				while(InSubCall);
			}
			finally { InCall = false; }
			return true;
		}
	}
}
