//==========================================================================================
//
//		OpenNETCF.Windows.Forms.HTMLViewer
//		Copyright (c) 2003, OpenNETCF.org
//
//		This library is free software; you can redistribute it and/or modify it under 
//		the terms of the OpenNETCF.org Shared Source License.
//
//		This library is distributed in the hope that it will be useful, but 
//		WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
//		FITNESS FOR A PARTICULAR PURPOSE. See the OpenNETCF.org Shared Source License 
//		for more details.
//
//		You should have received a copy of the OpenNETCF.org Shared Source License 
//		along with this library; if not, email licensing@opennetcf.org to request a copy.
//
//		If you wish to contact the OpenNETCF Advisory Board to discuss licensing, please 
//		email licensing@opennetcf.org.
//
//		For general enquiries, email enquiries@opennetcf.org or visit our website at:
//		http://www.opennetcf.org
//
//==========================================================================================

using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace ihook.PocketWidgets.HTML
//namespace OpenNETCF.Windows.Forms
{
	/// <summary>
	/// this is an helper class to have an embeded HTML control.
	/// doens't work very well, but does work.
	/// </summary>
	public class HTMLViewer : System.Windows.Forms.Control
	{
		/// <summary>
		/// Handle to the HTML Control window
		/// </summary>
		private IntPtr _hwndHtml;			
		/// <summary>
		/// App Instance
		/// </summary>
		private IntPtr _hAppInst = GetModuleHandle(null);	
		/// <summary>
		/// Handle to the dll
		/// </summary>
		private IntPtr _hInstHTMLCtrl = LoadLibrary("htmlview.dll");	
		/// <summary>
		/// Reference to the parent form that will contain the control
		/// </summary>
		private Form _parentForm=null;								
		/// <summary>
		/// HTML String to display
		/// </summary>
		private String _htmlString=null;								
		/// <summary>
		/// Left co-ordinant
		/// </summary>
		private IntPtr _htmlCtrl_Left = IntPtr.Zero;
		/// <summary>
		/// Top Co-Ordinant
		/// </summary>
		private IntPtr _htmlCtrl_Top = IntPtr.Zero;						
		/// <summary>
		/// Width
		/// </summary>
		private IntPtr _htmlCtrl_Width = IntPtr.Zero;					
		/// <summary>
		/// Height
		/// </summary>
		private IntPtr _htmlCtrl_Height = IntPtr.Zero;					
		/// <summary>
		/// Bounds of the control
		/// </summary>
		private Rectangle _bounds;



		#region ------- DLL Imports------------
		[DllImport("coredll.dll", EntryPoint="FindWindowW", SetLastError=true)]
		private static extern IntPtr FindWindowCE(string lpClassName, string lpWindowName);

		[DllImport("htmlview.dll", EntryPoint="InitHTMLControl", SetLastError=true)] 
		private static extern bool InitHTMLControl(IntPtr hinst);

		[DllImport("coredll.dll", EntryPoint="LoadLibraryW", SetLastError=true)] 
		private static extern IntPtr LoadLibrary(string lpszLib);

		[DllImport("coredll.dll", EntryPoint="CreateWindowExW", SetLastError=true)] 
		private static extern IntPtr CreateWindow(IntPtr dwExStyle,string lpClassName,string lpWindowName,
			IntPtr dwStyle,IntPtr x, IntPtr y, IntPtr nWidth, IntPtr nHeight,
			IntPtr hWndParent,IntPtr hMenu, IntPtr hInstance, IntPtr lpParam);

		[DllImport("coredll.dll", EntryPoint="MoveWindow", SetLastError=true)] 
		private static extern IntPtr MoveWindow(IntPtr hWnd, IntPtr x, IntPtr y, IntPtr nWidth, IntPtr nHeight);

		[DllImport("coredll.dll", EntryPoint="SetFocus", SetLastError=true)] 
		private static extern IntPtr SetFocusAPI(IntPtr hWnd);

		[DllImport("coredll.dll", EntryPoint="PostMessageW", SetLastError=true)] 
		private static extern IntPtr PostMessage(IntPtr hWnd, IntPtr wMsg, IntPtr wParam, IntPtr lParam);

		[DllImport("coredll.dll", EntryPoint="SendMessageW", SetLastError=true)] 
		private static extern IntPtr SendMessageString(IntPtr hWnd, IntPtr wMsg, IntPtr wParam, String lParam);

		[DllImport("coredll.dll", EntryPoint="SendMessageW", SetLastError=true)] 
		private static extern IntPtr SendMessageLong(IntPtr hWnd, IntPtr wMsg, IntPtr wParam, IntPtr lParam);

		[DllImport("coredll.dll", EntryPoint="GetWindow", SetLastError=true)] 
		private static extern IntPtr GetWindow(IntPtr hWnd, IntPtr wCmd);

		[DllImport("coredll.dll", EntryPoint="DestroyWindow", SetLastError=true)] 
		private static extern IntPtr DestroyWindow(IntPtr hWnd);
		
		[DllImport("coredll.dll", EntryPoint="ShowWindow", SetLastError=true)] 
		private static extern IntPtr ShowWindow(IntPtr hWnd, IntPtr nCmdShow);

		/// <summary>
		/// 
		/// </summary>
		/// <param name="lpModuleName"></param>
		/// <returns></returns>
		[DllImport("coredll.dll")]
		public static extern IntPtr GetModuleHandle(String lpModuleName);


		//hiding and showing window
		private const uint SW_SHOW = 5;
		private const uint SW_HIDE = 0;
		private const uint SW_NORMAL = 1;

		//HTMLview control messages
		private const uint  DTM_ADDTEXT = 1125;
		private const uint  DTM_ADDTEXTW = 1126;
		private const uint  DTM_SETIMAGE = 1127;
		private const uint  DTM_ENDOFSOURCE = 1128;
		private const uint  DTM_ANCHOR = 1129;
		private const uint  DTM_ANCHORW = 1130;
		private const uint  DTM_ENABLESHRINK = 1131;
		private const uint  DTM_FITTOWINDOW = 1132;
		private const uint  DTM_SCROLLINTOVIEW = 1133;
		private const uint  DTM_IMAGEFAIL = 1134;
		private const uint  DTM_REALLYDONE = 1135;
		private const uint  DTM_SELECTALL = 1135;
		private const uint  DTM_ISSELECTION = 1136;
		private const uint  DTM_CLEAR = 1137;
		private const uint  DTM_ENABLECLEARTYPE = 1138;
		private const uint  DTM_ENABLESCRIPTING = 1139;
		private const uint  DTM_ZOOMLEVEL = 1140;
		private const uint  DTM_LAYOUTWIDTH = 1141;
		private const uint  DTM_LAYOUTHEIGHT = 1142;
		// Windows messages
		private const uint  WM_VSCROLL = 0x115;
		private const uint  GW_CHILD = 0x5;
		private const uint  SB_PAGEDOWN = 0x3;
		private const uint  SB_PAGEUP = 0x2;
		private const uint  WM_COMMAND = 0x111;
		private const uint  WS_CHILD = 0x40000000;
		private const uint  WS_VISIBLE = 0x10000000;
		private const uint  WS_VSCROLL = 0x200000;
		private const uint  WS_CLIPSIBLINGS = 0x4000000;
		private const uint  WM_USER = 0x400;
		private const uint  WS_BORDER = 0x800000;
		private const uint  WM_SETTEXT = 0xC;
		private const uint  WS_HSCROLL = 0x100000;

		#endregion


		/// <summary>
		/// Default constructor
		/// </summary>
		public HTMLViewer()
		{			
		}


		/// <summary>
		/// Destructor
		/// </summary>
		~HTMLViewer()
		{
			DestroyWindow(this._hwndHtml);
		}

		/// <summary>
		/// Creates the HTML Control
		/// </summary>
		public void CreateHTMLControl()
		{
			if(this._parentForm== null)
				throw new Exception("Parent form cannot be null");

			this.CreateHTMLControl(FindWindowCE(null,this._parentForm.Text),this._htmlCtrl_Left ,this._htmlCtrl_Top ,this._htmlCtrl_Width ,this._htmlCtrl_Height);
		}

		/// <summary>
		/// Destroys the HTML Control
		/// </summary>
		public void DestroyHTMLControl()
		{
			DestroyWindow(this._hwndHtml);
		}


		/// <summary>
		/// Display the text the was set using the HtmlString property 
		/// </summary>
		public void DisplayText()
		{
			if(this._htmlString==null | this._htmlString=="")
				throw new Exception("HTML String not set! Nothing to display!");
			this.SetText(this._htmlString, true);	
		}

		/// <summary>
		/// Display the text using the parameter
		/// </summary>
		/// <param name="htmlText">The HTML string to display</param>
		public void DisplayText(String htmlText)
		{
			this.HtmlString = htmlText;
			this.DisplayText();
		}

		public override string Text
		{
			get { return _htmlString; }
			set { DisplayText(value); }
		}


		/// <summary>
		/// Resize the control using the appropriate co-ordinates
		/// </summary>
		/// <param name="left">left co-ord of the control</param>
		/// <param name="top">top  co-ord of the control</param>
		/// <param name="width">width of the control</param>
		/// <param name="height">height of the control</param>
		public void ResizeHTMLControl(int left, int top, int width, int height)
		{
			this.Left = left;
			this.Top = top;
			this.Width = width;
			this.Height = height;
			this.ResizeHTMLControl();
		}
		/// <summary>
		/// Resize the HTML Control using bounds co-ordinants
		/// </summary>
		/// <param name="bounds">Rectangle the control will use</param>
		public void ResizeHTMLControl(Rectangle bounds)
		{
			this.Bounds = bounds;
			this.ResizeHTMLControl();
		}
		/// <summary>
		/// Resize the HTML Control
		/// </summary>
		public void ResizeHTMLControl()
		{
			MoveWindow(this._hwndHtml,this._htmlCtrl_Left,this._htmlCtrl_Top,this._htmlCtrl_Width,this._htmlCtrl_Height);
		}

		/// <summary>
		/// Set the bounds of the HTML Control
		/// </summary>
		public new Rectangle Bounds
		{
			get
			{
				return this._bounds;
			}
			set
			{
				this._bounds = value;
				this._htmlCtrl_Height = (IntPtr)value.Height;
				this._htmlCtrl_Left = (IntPtr)value.Left;
				this._htmlCtrl_Top = (IntPtr)value.Top;
				this._htmlCtrl_Width = (IntPtr)value.Width;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public IntPtr ApplicationInstance
		{
			set
			{
				this._hAppInst = value;
			}
		}

		/// <summary>
		/// The left co-ordinant of the control
		/// </summary>
		public new int Left
		{
			get
			{
				return (int)this._htmlCtrl_Left;
			}
			set
			{
				this._htmlCtrl_Left = (IntPtr)value;
			}
		}

		/// <summary>
		/// The top co-ordinant of the control
		/// </summary>
		public new int Top
		{
			get
			{
				return (int)this._htmlCtrl_Top;
			}
			set
			{
				this._htmlCtrl_Top = (IntPtr)value;
			}
		}

		/// <summary>
		/// The width of the control
		/// </summary>
		public new int Width
		{
			get
			{
				return (int)this._htmlCtrl_Width;
			}
			set
			{
				this._htmlCtrl_Width = (IntPtr)value;
			}
		}

		/// <summary>
		/// The height of the control
		/// </summary>
		public new int Height
		{
			get
			{
				return (int)this._htmlCtrl_Height;
			}
			set
			{
				this._htmlCtrl_Height = (IntPtr)value;
			}
		}

		/// <summary>
		/// The parent form that will contain the control
		/// </summary>
		public Form ParentForm
		{
			get
			{
				return this._parentForm;
			}
			set
			{
				this._parentForm= value;
			}
		}

		/// <summary>
		/// The HTML String that will be displayed
		/// </summary>
		public String HtmlString
		{
			get
			{
				return this._htmlString;
			}
			set
			{
				this._htmlString = value;

			}
		}



		private void CreateHTMLControl(IntPtr hWndParent, IntPtr iLeft, IntPtr iTop, IntPtr iWidth, IntPtr iHeight)
		{
			IntPtr dwStyle = (IntPtr)(WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_BORDER) ;

			//Init the library
			InitHTMLControl(this._hAppInst);

			//Create the window
			this._hwndHtml = CreateWindow(IntPtr.Zero ,"DISPLAYCLASS","HTMLV",dwStyle,iLeft,iTop,iWidth,iHeight,hWndParent,IntPtr.Zero,this._hAppInst,IntPtr.Zero);
			SetFocusAPI(this._hwndHtml);
			PostMessage(this._hwndHtml,(IntPtr)DTM_ENABLESHRINK,IntPtr.Zero,(IntPtr)1);
			PostMessage(this._hwndHtml,(IntPtr)DTM_ENABLESCRIPTING,IntPtr.Zero,(IntPtr)1);
		}

		private void SetText(String sData, bool bClearType)
		{
			IntPtr ret = SendMessageString(this._hwndHtml,(IntPtr)WM_SETTEXT,IntPtr.Zero,"");
			if(bClearType)
				ret = SendMessageLong(this._hwndHtml,(IntPtr)DTM_ENABLECLEARTYPE,IntPtr.Zero,(IntPtr)1);

			ret = SendMessageLong(this._hwndHtml, (IntPtr)DTM_ENABLESCRIPTING, IntPtr.Zero,(IntPtr)1);
			ret = SendMessageString(this._hwndHtml,(IntPtr) DTM_ADDTEXTW, (IntPtr)0, sData);
			ret = SendMessageLong(this._hwndHtml,(IntPtr) DTM_ENDOFSOURCE, IntPtr.Zero, IntPtr.Zero);
		}
		private string GetFilePath()
		{
			return System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase;
		}
	}
}
