#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ihook.PocketWidgets
{
	public class IconButton : SmartControl
	{
		#if NETCFDESIGNTIME
		[
		System.ComponentModel.Category("Appearance "),
		System.ComponentModel.Description("The icon to be displayed inside the button")
		]
		#endif
		public virtual Icon Icon
		{
			get { return icon; }
			set
			{
				icon = value;
				this.Invalidate();
			}
		}
		Icon icon;

		public IconButton()
		{
			Size = new Size(20, 20);
		}
		public IconButton(Icon ic) : this()
		{
			Icon = ic;
		}

		protected override void OnPaintBackground(System.Windows.Forms.PaintEventArgs e)
		{
		}

		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			base.OnPaint(e);

			Brush b;
			if(Enabled)
				b = pressed ? AppDefaults.WhiteBrush : AppDefaults.DisabledBackgroundBrush;
			else
				b = AppDefaults.DarkBrush;
			e.Graphics.FillRectangle(b, ClientRectangle);

			Pen p = AppDefaults.BorderPen;
			e.Graphics.DrawRectangle(p, 0, 0, Width-1, Height-1);

			if(icon != null)
			{
				int x = (Width - 16) / 2;
				int y = (Height - 16) / 2;
				e.Graphics.DrawIcon(icon, x, y);
			}
		}

		bool pressed = false;
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			Focus();
			if(Enabled)
			{
				pressed = true;
				Invalidate();
			}
		}
		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp (e);
			if(Enabled)
			{
				pressed = false;
				Invalidate();
			}
		}
	}
}