#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion

//#define AS_HACK
#define AS_CONTROL

using System;
using System.Collections;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets
{
	/// <summary>
	/// a "dialog" class, which is just a control showing itself
	/// at the top z-order. (to avoid hiding the main windows,
	/// as do standart dialog boxes using ShowDialog() ;-(
	/// </summary>
	// can't be a form, form.Show() hide the main window !
	// ggrrmmrrbblll ....
#if AS_HACK
	public class PocketDialog
#elif AS_CONTROL
	public class PocketDialog : SmartControl
#endif
	{
		Form _body;
		/// <summary>
		/// construct a PocketDialog with the controls of an existing
		/// form (created with the desigher ?)
		/// </summary>
		/// <param name="body"></param>
		public PocketDialog(Form body)
		{
			_body = body;
#if AS_CONTROL
			foreach(Control c in body.Controls)
			{
				Controls.Add(c);
				Point p = c.Location;
				c.Location = new Point(p.X+1, p.Y+25);
			}
			Text = body.Text;
			Size = new Size(body.Width+2, body.Height + 25);
#endif
#if AS_HACK
			int style = WinAPI.GetWindowLong(Handle, WinAPI.GetWindowLongParam.GWL_STYLE);
			style = (int)(
				WinAPI.WindowStyle.WS_CAPTION 
				| WinAPI.WindowStyle.WS_POPUP
				);
			WinAPI.SetWindowLong(Handle, WinAPI.GetWindowLongParam.GWL_STYLE, style);
			body.FormBorderStyle = FormBorderStyle.None;
			body.Size = new Size(body.Width+2, body.Height + 25);
			body.Deactivate += new EventHandler(body_Deactivate);
#endif
			ScreenLocation = new Point(10, 40);
		}
#if AS_HACK
		private void body_Deactivate(object sender, EventArgs e)
		{
			if(modalShow)
				_body.BringToFront();
		}

		IntPtr handle = IntPtr.Zero;
		public IntPtr Handle
		{
			get
			{
				if(handle == IntPtr.Zero)
				{
					string text = Text;
					Text = text+"                       --#%$@!#$#!!--";
					handle = WinAPI.FindWindow(null, Text);
					Text = text;
				}
				return handle;
			}
		}
		public virtual string Text
		{
			get { return _body.Text; }
			set { _body.Text = value; }
		}
#endif

#if AS_CONTROL
		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);
			e.Graphics.FillRectangle(AppDefaults.BlueBrush, 0, 0, Width, 24);
			e.Graphics.DrawRectangle(AppDefaults.TextPen, 0, 0, Width-1, Height-1);
			e.Graphics.DrawLine(AppDefaults.TextPen, 0, 24, Width, 24);
			e.Graphics.DrawString(Text, AppDefaults.BoldFont, AppDefaults.WhiteBrush, 4, 4);
		}
		protected Point MouseDownPoint;
		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			Capture = true;
			MouseDownPoint = new Point(e.X, e.Y);
		}
		protected override void OnMouseMove(MouseEventArgs e)
		{
			base.OnMouseMove (e);
			if(Capture)
			{
				int dx = e.X - MouseDownPoint.X;
				int dy = e.Y - MouseDownPoint.Y;
				Point p = Location;
				p.X += dx;
				p.Y += dy;
				Location = p;
			}
		}
		protected override void OnMouseUp(MouseEventArgs e)
		{
			base.OnMouseUp (e);
			Capture = false;
		}
#endif

		public Point ScreenLocation
		{
			get
			{
#if AS_HACK
				return _body.Location;
#elif AS_CONTROL
				Point p = Location;
				return new Point(p.X, p.Y-20);
#endif
			}
			set
			{
#if AS_HACK
				_body.Location = value;
#elif AS_CONTROL
				Location = new Point(value.X, value.Y+20);
#endif
			}
		}

		public bool SoundAlert = true;
		public WinAPI.BeepSound BeepSound =  WinAPI.BeepSound.Question;

		DialogResult result = DialogResult.No;
		public DialogResult DialogResult
		{
			get { return result; }
			set 
			{ 
				result = value; 
				_body.DialogResult = value;
			}
		}

		public bool DialogVisible
		{
			get
			{
#if AS_CONTROL
				return Parent != null;
#elif AS_HACK
				return _body.Visible;
#endif
			}
		}

		public void Show(Control c)
		{
			if(PopupPanel.SinglePopup != null)
				PopupPanel.SinglePopup.PopDown();

#if AS_CONTROL
			Parent = c.TopLevelControl;
			BringToFront();
#elif AS_HACK
			_body.Show();
#endif
			if(SoundAlert)
				WinAPI.Beep(BeepSound);
		}

		/// <summary>
		/// this function seems less likely to hide the main Form than 
		/// ShowDialog(), which is a real hassel as no function seems
		/// to be able to show it again, after  that ... ???!!!
		/// </summary>
		/// <returns></returns>
		public virtual DialogResult ShowModal(Control c)
		{
			DialogResult = DialogResult.Cancel;
			modalShow = true;
			Show(c);
			ModalExcludeFilter.Shared.Push(this);
			try
			{
				while(modalShow && DialogVisible)
					ApplicationEx.Pump();
			}
			finally 
			{
				ModalExcludeFilter.Shared.Pop();
			}
			EndShow();

			return DialogResult;
		}
		bool modalShow = false;

		static PocketDialog()
		{
			ApplicationEx.AddMessageFilter(ModalExcludeFilter.Shared);
		}

		/// <summary>
		/// this is to be called by some dialog event handler, 
		/// to close the dialog and return from ShowModal() call
		/// </summary>
		public virtual void EndShow()
		{
			if(PopupPanel.SinglePopup != null)
				PopupPanel.SinglePopup.PopDown();

			modalShow = false;
#if AS_CONTROL
			Parent = null;
#elif AS_HACK
			_body.Hide();
#endif
		}
		/// <summary>
		/// this is to be called by some dialog event handler, 
		/// to close the dialog and return from ShowModal() call
		/// </summary>
		/// <param name="dr">the value to return from ShowModal()</param>
		public virtual void EndModal(DialogResult dr)
		{
			DialogResult = dr;
			EndShow();
		}
	}

	#region message filters

	/// <summary>
	/// an attempt of a modal filter
	/// </summary>
	public abstract class ModalExcludeFilter : IMessageFilter
	{
		IntPtr hwndCurrent = IntPtr.Zero;
		Stack windows = new Stack();
		public ModalExcludeFilter()
		{
		}
		public void Push(PocketDialog p)
		{
			if(p == null)
				throw new ArgumentNullException();
			IntPtr hwnd = p.Handle;
			windows.Push(hwndCurrent);
			hwndCurrent = hwnd;
		}
		public void Pop()
		{
			if(windows.Count > 0)
				hwndCurrent = (IntPtr) windows.Pop();
		}
		protected IntPtr CurrentWindow
		{
			get { return hwndCurrent; }
		}
		public abstract bool PreFilterMessage(ref Message m);

		protected bool InHierarchy(IntPtr top, IntPtr hwnd)
		{
			if(top.Equals(IntPtr.Zero))
				return false;
			if(top.Equals(hwnd))
				return true;
			return WinAPI.IsChild(top, hwnd);
		}
		protected bool IsPopup(IntPtr hwnd)
		{
			hwnd = WinAPI.GetTopMostParent(hwnd);
			int style = WinAPI.GetWindowLong(hwnd, WinAPI.GetWindowLongParam.GWL_STYLE);
			//return (style & (int) WinAPI.WindowStyle.WS_POPUPWINDOW) == (int) WinAPI.WindowStyle.WS_POPUPWINDOW;
			return (style & (int) WinAPI.WindowStyle.WS_POPUP) == (int) WinAPI.WindowStyle.WS_POPUP;
		}

		static ModalExcludeFilter shared;
		public static ModalExcludeFilter Shared
		{
			get
			{
				if(shared == null)
#if AS_HACK
					shared = new ModalExcludeWindow();
#elif AS_CONTROL
					shared = new ModalExcludeControl();
#endif
				return shared;
			}
		}
	}
	public class ModalExcludeControl : ModalExcludeFilter
	{
		public override bool PreFilterMessage(ref Message m)
		{
			IntPtr hwnd = CurrentWindow;

			if(hwnd == IntPtr.Zero)
				return false;
			if(m.Msg == WinAPI.WM_PAINT)
				return false;
			if(InHierarchy(hwnd, m.HWnd))
				return false;
			if(WinAPI.GetTopMostParent(hwnd) != WinAPI.GetTopMostParent(m.HWnd))
				return false;

			if(m.Msg == WinAPI.WM_LBUTTONDOWN)
				WinAPI.Beep(WinAPI.BeepSound.Hand);

			return true;
		}
	}
	public class ModalExcludeWindow : ModalExcludeFilter
	{
		public override bool PreFilterMessage(ref Message m)
		{
			IntPtr hwnd = CurrentWindow;

			if(hwnd == IntPtr.Zero)
				return false;
			if(m.Msg == WinAPI.WM_PAINT)
				return false;
			if(InHierarchy(hwnd, m.HWnd))
				return false;
			if(IsPopup(m.HWnd))
				return false;

			if(m.Msg == WinAPI.WM_LBUTTONDOWN)
				WinAPI.Beep(WinAPI.BeepSound.Hand);

			return true;
		}
	}
	#endregion
}
