#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ihook.PocketWidgets
{
	/// <summary>
	/// this is a popup panel, which show itself around a control.
	/// I don't use modal dialog box, as they bug ...
	/// </summary>
	public class PopupPanel : ScrollPane
	{
		static internal PopupPanel SinglePopup;

		Form parentForm;
		ContentAlignment align = ContentAlignment.TopRight;
		public PopupPanel()
		{
			parentForm = new Form();
			parentForm.Deactivate += new EventHandler(parentForm_Deactivate);
			Parent = parentForm;
			parentForm.FormBorderStyle = FormBorderStyle.None;
			parentForm.Menu = null;
		}
		public PopupPanel(ContentAlignment ca) : this()
		{
			align = ca;
		}
		private void parentForm_Deactivate(object sender, EventArgs e)
		{
			PopDown();
		}

		public event EventHandler ShownUp;
		public event EventHandler ClosedUp;

		public bool Popping 
		{ 
			get { return parentForm.Visible; } 
		}

		public virtual void PopUp(Control c)
		{
			if(SinglePopup != null && SinglePopup != this)
				SinglePopup.PopDown();

			if(Popping)
				return;

			Point p = c.PointToScreen(new Point(0, 0));
			Control top = c.TopLevelControl;

			Size sControl = c.Size;
			Size sPopup = Size;
			Size sSpace = new Size(240, 320);

			int y = p.Y+sControl.Height-1;
			if(p.Y+sControl.Height+sPopup.Height>sSpace.Height
				&& p.Y - sPopup.Height >= 0)
				y = p.Y - sPopup.Height + 1;

			int x;
			switch(align)
			{
				case ContentAlignment.TopCenter:
					x = p.X + (sControl.Width-sPopup.Width)/2;
					break;
				case ContentAlignment.TopLeft:
					x = p.X;
					break;
				default:
				case ContentAlignment.TopRight:
					x = p.X + sControl.Width - sPopup.Width;
					break;
			}
			if(x+sPopup.Width > sSpace.Width)
				x = sSpace.Width - sPopup.Width;
			if(x < 0)
				x = 0;

			parentForm.Size = Size;
			parentForm.Location = new Point(x, y);
			parentForm.Show();

			SinglePopup = this;
			if(ShownUp != null)
				ShownUp(this, null);
		}
		public virtual void PopDown()
		{
			SinglePopup = null;
			parentForm.Hide();

			if(ClosedUp != null)
				ClosedUp(this, null);
		}
	}
}
