#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Data;

namespace ihook.PocketWidgets.Table
{
	/// <summary>
	/// a convenient ITableModel meant to map a DataView in order to
	/// be shown in the TableControl
	/// </summary>
	public class DataTableModel : ITableModel
	{
		public DataTableModel(DataView dv) : this()
		{
			DataView = dv;
		}
		public DataTableModel(DataTable dt) : this()
		{
			DataView = dt.DefaultView;
		}
		public DataTableModel()
		{
		}

		DataView datas;
		public virtual DataView DataView
		{
			get { return datas; }
			set 
			{ 
				datas = value;
				Fire(TableModelEvent.EType.Structure);
			}
		}

		public virtual int NumRows
		{
			get
			{
				if(datas == null)
					return 0;
				return datas.Count;
			}
		}
		public virtual int NumColumns
		{
			get
			{
				if(datas == null)
					return 0;
				return datas.Table.Columns.Count;
			}
		}

		public virtual string GetColumnTitle(int col)
		{
			return datas.Table.Columns[col].ColumnName;
		}
		public virtual Type GetColumnType(int col)
		{
			return datas.Table.Columns[col].DataType;
		}

		public virtual object this[int row, int col]
		{
			get { return datas[row].Row[col]; }
			set 
			{ 
				if(value == null)
					value = DBNull.Value;
				datas[row].Row[col] = value; 
				Fire(TableModelEvent.EType.RowUpdated, row, row, col);
			}
		}
		public virtual bool IsEditable(int row, int col)
		{
			if(!datas.AllowEdit)
				return false;
			if(datas.Table.Columns[col].ReadOnly)
				return false;
			return true;
		}

		#region -- event functions --

		public virtual void Fire()
		{
			Fire(TableModelEvent.EType.All, 0, NumRows);
		}
		public virtual void Fire(TableModelEvent.EType etype)
		{
			Fire(etype, 0, NumRows);
		}
		public virtual void Fire(TableModelEvent.EType etype, int row)
		{
			Fire(etype, row, row);
		}
		public virtual void Fire(TableModelEvent.EType etype, int row0, int row1)
		{
			if(Changed == null)
				return;
			Changed(new TableModelEvent(this, etype, row0, row1));
		}
		public virtual void Fire(TableModelEvent.EType etype, int row0, int row1, int column)
		{
			if(Changed == null)
				return;
			Changed(new TableModelEvent(this, etype, row0, row1, column));
		}
		public event TableEventHandler Changed;

		#endregion
	}
}
