#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Utils;

using IconManager = ihook.PocketWidgets.Icons.IconManager;

namespace ihook.PocketWidgets.Table.Renderers
{
	/// <summary>
	/// Summary description for BoolRenderer.
	/// </summary>
	public class BoolRenderer : ICellRenderer
	{
		static BoolRenderer()
		{
			Assembly a = typeof(BoolRenderer).Assembly;
		}
		public BoolRenderer()
		{
		}
		public BoolRenderer(string sTrue, string sFalse) : this()
		{
			TrueText = sTrue;
			FalseText = sFalse;
		}

		Icon ico_checked, ico_unchecked;
		public Icon IconChecked
		{
			get 
			{
				if(ico_checked == null)
					ico_checked = IconManager.GetIcon("checked.ico");
				return ico_checked;
			}
			set { ico_checked = value; }
		}
		public Icon IconUnchecked
		{
			get 
			{
				if(ico_unchecked == null)
					ico_unchecked = IconManager.GetIcon("unchecked.ico");
				return ico_unchecked;
			}
			set { ico_unchecked = value; }
		}
		#region ICellRenderer Members

		public void PaintCell(Graphics g, TableControl tc, Rectangle r, int row, int col, object val)
		{
			TextTableRenderer.PaintBackground(g, tc, r, row);
			Paint(g, r, tc.Model[row, col]);
		}

		Brush textBrush;
		public Brush TextBrush
		{
			get 
			{
				if(textBrush == null)
					textBrush = AppDefaults.TextBrush;
				return textBrush;
			}
			set { textBrush = value; }
		}

		public string TrueText, FalseText, NullText = TextTableRenderer.NULL_TEXT;
		public virtual void Paint(Graphics g, Rectangle r, object val)
		{
			Font f = AppDefaults.Font;
			SizeF s1 = TrueText==null ? new SizeF() : g.MeasureString(TrueText, f);
			SizeF s2 = FalseText==null ? new SizeF() : g.MeasureString(FalseText, f);
			SizeF s3 = NullText==null ? new SizeF() : g.MeasureString(NullText, f);
			int w = 0;
			if(s1.Width > w)
				w = (int) s1.Width;
			if(s2.Width > w)
				w = (int) s2.Width;
			//if(s3.Width > w)
			//	w = (int) s3.Width;

			w += 18;
			int dx = (r.Width - w) / 2;
			if(val == null || val == DBNull.Value)
			{
				if(NullText != null)
					g.DrawString(NullText, f, TextBrush, r.Right - s3.Width - 1, r.Y+1);
			}
			else if((bool) val == true)
			{
				g.DrawIcon(IconChecked, r.X+dx, r.Y+1);
				if(TrueText != null)
					g.DrawString(TrueText, f, TextBrush, r.X+dx, r.Y+1);
			}
			else
			{
				g.DrawIcon(IconUnchecked, r.X+dx, r.Y+1);
				if(FalseText != null)
					g.DrawString(FalseText, f, TextBrush, r.X+dx, r.Y+1);
			}
		}

		#endregion

		#region ICell Members

		public void OnMouseDown(TableControl tc, int row, int col, MouseEventArgs e)
		{
			// TODO:  Add BoolRenderer.OnMouseDown implementation
		}

		#endregion
	}
}
