#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Editors;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Table.Renderers
{
	/// <summary>
	/// Summary description for DateTimeRenderer.
	/// </summary>
	public class DateTimeRenderer : TextTableRenderer
	{
		DateEditorFormat type;

		public DateTimeRenderer(DateEditorFormat dt, ContentAlignment ca)
			: base(ca)
		{
			type = dt;
		}
		public DateTimeRenderer(DateEditorFormat dt)
			: this(dt, ContentAlignment.TopRight)
		{
		}
		public DateTimeRenderer()
			: this(DateEditorFormat.ShortDate, ContentAlignment.TopRight)
		{
		}

		public override void PaintCell(Graphics g, TableControl tc, Rectangle r, int row, int col, object val)
		{
			if(val != null && val != DBNull.Value)
			{
				DateTime dt = (DateTime) val;
				string text;
				switch(type)
				{
					case DateEditorFormat.DateTime:
						text = dt.ToLongTimeString();
						break;
					case DateEditorFormat.LongDate:
						text = dt.ToLongDateString();
						break;
					default:
					case DateEditorFormat.ShortDate:
						text = dt.ToShortDateString();
						break;
					case DateEditorFormat.Time:
						text = dt.ToShortTimeString();
						break;
				}
				base.PaintCell(g, tc, r, row, col, text);
			}
			else
				base.PaintCell(g, tc, r, row, col, val);
		}
	}
}
