#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Table.Renderers
{
	/// <summary>
	/// Summary description for TextTableRenderer.
	/// </summary>
	public class TextTableRenderer : ICellRenderer
	{
		ContentAlignment align;

		public TextTableRenderer() : this(ContentAlignment.TopLeft)
		{
		}
		public TextTableRenderer(ContentAlignment ca)
		{
			align = ca;
		}
		
		Brush textBrush;
		public Brush TextBrush
		{
			get 
			{
				if(textBrush == null)
					textBrush = AppDefaults.TextBrush;
				return textBrush;
			}
			set { textBrush = value; }
		}

		#region ICellRenderer Members

		public static void PaintBackground(Graphics g, TableControl tc, Rectangle r, int row)
		{
			Brush b = row%2==0 ? AppDefaults.WhiteBrush : AppDefaults.LightBrush;
			if(tc.SelectedRow == row)
				b = AppDefaults.SelectionBrush;
			g.FillRectangle(b, r);
		}

		public const string NULL_TEXT = "(null)";
		public virtual void PaintCell(Graphics g, TableControl tc, Rectangle r, int row, int col, object val)
		{
			PaintBackground(g, tc, r, row);

			r.X += 1;
			r.Y += 1;
			r.Width -= 2;
			r.Height -= 2;
			string text = (val==null || val==DBNull.Value) ? NULL_TEXT : val.ToString();
			Size size = g.MeasureString(text, tc.Font).ToSize();
			if(size.Width < r.Width)
				switch(align)
				{
					case ContentAlignment.TopCenter:
						r.X += (r.Width - size.Width) / 2;
						break;
					default:
					case ContentAlignment.TopLeft:
						// nothing ;-)
						break;
					case ContentAlignment.TopRight:
						r.X = r.Right - size.Width;
						break;
				}
			g.DrawString(text, tc.Font, TextBrush, r.X, r.Y);
		}
		public virtual void OnMouseDown(TableControl tc, int row, int col, MouseEventArgs e)
		{
		}

		#endregion
	}
}
