﻿#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Table
{
	/// <summary>
	/// This is the main mathematical body.
	/// the position function always work in the same mathematical space,
	/// irregardless of the scroll state of the view. the cell size include
	/// it's 1x1 border (at the right bottom) but doesn't include the 1x1 initial
	/// offset due to the border
	/// </summary>
	public class TableBody : ICellRenderer
	{
		public TableBody()
		{
		}
		internal TableBody(TableControl t) : this()
		{
			Table = t;
		}

		Image background;
		public Image BackgroundImage
		{
			get { return background; }
			set 
			{
				background = value;
				Table.Invalidate();
			}
		}
		/// <summary>
		/// clear all temporary user setting such as customized 
		/// row &amp; columns size.
		/// </summary>
		public virtual void Reset()
		{
			colWidth = null;
			rowHeight = null;
		}
		TableControl table;
		internal TableControl Table
		{
			set 
			{ 
				table = value; 
				Reset();
			}
			get { return table; }
		}

		int defaultRowHeight = -1;
		public int DefaultRowHeight
		{
			get 
			{ 
				if(defaultRowHeight < 1) 
				{
//					Font f = AppDefaults.Font;
//					defaultRowHeight = (int) f.Size + 2;
					defaultRowHeight = 18;
				}
				return defaultRowHeight; 
			}
			set 
			{ 
				defaultRowHeight = value;
			}
		}
		int defaultColWidth = -1;
		public int DefaultColWidth
		{
			get 
			{ 
				if(defaultColWidth < 1) 
					defaultColWidth = 50;
				return defaultColWidth; 
			}
			set 
			{ 
				defaultColWidth = value;
			}
		}

		public void DeleteRows(int row0, int nRow)
		{
			if(nRow < 1 || rowHeight == null)
				return;
			Hashtable newRowHeight = new Hashtable();
			foreach(int row in rowHeight.Keys)
			{
				if(row < row0)
					newRowHeight[row] = rowHeight[row];
				else if (row >= row0 + nRow)
					newRowHeight[row - nRow] = rowHeight[row];
			}
			rowHeight = newRowHeight;
		}
		public void InsertRows(int row0, int nRow)
		{
			if(nRow < 1 || rowHeight == null)
				return;
			Hashtable newRowHeight = new Hashtable();
			foreach(int row in rowHeight.Keys)
			{
				if(row < row0)
					newRowHeight[row] = rowHeight[row];
				else
					newRowHeight[row + nRow] = rowHeight[row];
			}
			rowHeight = newRowHeight;
		}

		Hashtable colWidth, rowHeight;
		public int GetColWidth(int col)
		{
			if(colWidth != null)
			{
				object o = colWidth[col];
				if(o != null)
					return (int) o;
			}
			return DefaultColWidth;
		}
		public void SetColWidth(int col, int width)
		{
			int dw = DefaultColWidth;
			if(width < 1)
				width = 1;
			if(width == dw)
			{
				if(colWidth == null)
					return;
				colWidth.Remove(col);
				if(colWidth.Count == 0)
					colWidth = null;
			}
			else
			{
				if(colWidth == null)
					colWidth = new Hashtable();
				colWidth[col] = width;
			}
		}
		public int GetRowHeight(int row)
		{
			if(rowHeight != null)
			{
				object o = rowHeight[row];
				if(o != null)
					return (int) o;
			}
			return DefaultRowHeight;
		}
		public void SetRowHeight(int row, int height)
		{
			int dh = DefaultRowHeight;
			if(height < 1)
				height = 1;
			if(height == dh)
			{
				if(rowHeight == null)
					return;
				rowHeight.Remove(row);
				if(rowHeight.Count == 0)
					rowHeight = null;
			}
			else
			{
				if(rowHeight == null)
					rowHeight = new Hashtable();
				rowHeight[row] = height;
			}
		}

		public int RowAtPoint(Point p)
		{
			int row, col;
			CellAtPoint(p, out row, out col);
			return row;
		}
		public int ColAtPoint(Point p)
		{
			int row, col;
			CellAtPoint(p, out row, out col);
			return col;
		}
		public void CellAtPoint(Point p, out int row, out int col)
		{
			// 1st get an estimate
			int dw = DefaultColWidth;
			int dh = DefaultRowHeight;
			if(p.X<0)
				col = -1;
			else
				col = p.X / dw;
			if(p.Y<0)
				row = -1;
			else
				row = p.Y / dh;

			// then fine tuning
			Point co = GetCellOffset(row, col);
			Size cs;
			while(p.X<co.X && col > -1) 
			{
				col --;
				cs = GetCellSize(row, col);
				co.X -= cs.Width;
			}
			while(p.Y<co.Y && row > -1) 
			{
				row --;
				cs = GetCellSize(row, col);
				co.Y -= cs.Height;
			}
			for(cs = GetCellSize(row, col); p.X >= co.X+cs.Width;)
			{
				col ++;
				co.X += cs.Width;
				cs = GetCellSize(row, col);
			}
			for(cs = GetCellSize(row, col); p.Y >= co.Y+cs.Height;)
			{
				row ++;
				co.Y += cs.Height;
				cs = GetCellSize(row, col);
			}
		}

		public Size GetCellSize(int row, int col)
		{
			int w;
			int h;
			object o;

			if(rowHeight != null && (o = rowHeight[row]) != null)
				h = (int) o;
			else
				h = DefaultRowHeight;

			if(colWidth != null && (o = colWidth[col]) != null)
				w = (int) o;
			else
				w = DefaultColWidth;

			return new Size(w, h);
		}
		/// <summary>
		/// this is the offset in the mathematical body viewing
		/// space, without the offset du to the header or
		/// the scrolling
		/// </summary>
		/// <param name="row">the row you want to know where it is</param>
		/// <param name="col">the column you want to know where it is</param>
		/// <returns>return the offset of the top right corner relatively to
		/// the coordinate of the top right corner of the table</returns>
		public Point GetCellOffset(int row, int col)
		{
			int dw = DefaultColWidth;
			int dh = DefaultRowHeight;
			int x = col * dw;
			int y = row * dh;
			if(row > 0 && rowHeight != null)
			{
				foreach(int aRow in rowHeight.Keys)
					if(aRow < row && aRow > -1)
						y += (int) rowHeight[aRow] - dh;
			}
			else if(row < 0 && rowHeight != null)
			{
				foreach(int aRow in rowHeight.Keys)
					if(aRow >= row && aRow < 0)
						y -= (int) rowHeight[aRow] - dh;
			}
			if(col > 0 && colWidth != null)
			{
				foreach(int aCol in colWidth.Keys)
					if(aCol < col && aCol > -1)
						x += (int) colWidth[aCol] - dw;
			}
			else if(col < 0 && colWidth != null)
			{
				foreach(int aCol in colWidth.Keys)
					if(aCol >= col && aCol < 0)
						x -= (int) colWidth[aCol] - dw;
			}
			return new Point(x, y);
		}

		public virtual void Paint(PaintEventArgs e)
		{
			TableControl tc = Table;
			if(tc == null)
				return;
			ITableModel model = tc.Model;
			Point p0 = tc.BodyOffset;
			Point p00 = p0;
			
			Point p1 = tc.ScrollableLocation;
			p0.X += p1.X;
			p0.Y += p1.Y;

			if(model == null)
			{
				PaintBackground(p0, e.Graphics, e.ClipRectangle);
				return;
			}

			int nc = model.NumColumns;
			int nr = model.NumRows;

			// draw grid & first line
			int iMin, iMax, jMin, jMax;
			tc.GetBordingCells(e.ClipRectangle, out jMin, out jMax, out iMin, out iMax);
			if(iMax >= nc)
			{
				Point p = GetCellOffset(0, nc);
				int xMin = p.X+p0.X;
				Rectangle r = new Rectangle(xMin, p00.Y+1, tc.Width, tc.Height);
				PaintBackground(p0, e.Graphics, r);
			}
			if(jMax >= nr)
			{
				Point p = GetCellOffset(nr, nc);
				int yMin = p.Y+p0.Y;
				Rectangle r = new Rectangle(p00.X+1, yMin, p.X, tc.Height);
				PaintBackground(p0, e.Graphics, r);
			}
			// set the min max values to exisiting cells
			if(iMin < 0)
				iMin = 0;
			if(iMax >= nc)
				iMax = nc-1;
			if(jMin < 0)
				jMin = 0;
			if(jMax >= nr)
				jMax = nr-1;

			Pen pen = AppDefaults.LightPen;
			Rectangle clip = e.ClipRectangle;
			for(int j=jMin; j<=jMax; j++)
				for(int i=iMin; i<=iMax; i++) 
				{
					Point p = GetCellOffset(j, i);
					Size s = GetCellSize(j, i);
					Rectangle r = new Rectangle(p0.X+p.X, p0.Y+p.Y, s.Width, s.Height);
					Rectangle rClip = r;
					rClip.Intersect(clip);
					e.Graphics.Clip = new Region(rClip);
					tc.Renderers[j,i].PaintCell(e.Graphics, Table, r, j, i, model[j, i]);

					if(table.ShowRowLine)
						e.Graphics.DrawLine(pen, p0.X+p.X, p0.Y+p.Y, p0.X+p.X+s.Width, p0.Y+p.Y);
					if(table.ShowColLine)
						e.Graphics.DrawLine(pen, p0.X+p.X, p0.Y+p.Y, p0.X+p.X, p0.Y+p.Y+s.Height);
				}
		}
		/// <summary>
		/// call by the Paint() method to draw the empty background area
		/// </summary>
		/// <param name="g"></param>
		/// <param name="r"></param>
		protected virtual void PaintBackground(Point p0, Graphics g, Rectangle r)
		{
			if(background != null)
			{
				int w = background.Width;
				int h = background.Height;
				if(w > 0 && h > 0)
				{
					g.Clip = new Region(r);
					int x = (r.X / w) * w;
					int y = (r.Y / h) * h;
					for(int i=x; i<r.Right; i+=w)
						for(int j=y; j<r.Bottom; j+= h)
							g.DrawImage(background, x, y);
					return;
				}
			}
			g.FillRectangle(AppDefaults.EmptyBrush, r);
		}

		public static Point CellBottomRight(Point p, Size s)
		{
			p.X += s.Width;
			p.Y += s.Height;
			return p;
		}
		public Size Size
		{
			get
			{
				TableControl tc = Table;
				if(tc == null)
					return new Size();
				ITableModel model = tc.Model;
				if(model == null)
					return new Size();

				int nc = model.NumColumns;
				int nr = model.NumRows;
				Point pbr = CellBottomRight(GetCellOffset(nr-1, nc-1), GetCellSize(nr-1, nc-1));
				return new Size(pbr.X, pbr.Y);
			}
		}
		public virtual ICellRenderer GetCellRenderer(int row, int col)
		{
			return this;
		}

		#region  ICellRenderer
		public virtual void PaintCell(Graphics g, TableControl tc, Rectangle r, int row, int col, object val)
		{
			Brush b = row%2==0 ? AppDefaults.WhiteBrush : AppDefaults.LightBrush;
			if(tc.SelectedRow == row)
				b = AppDefaults.SelectionBrush;
			g.FillRectangle(b, r);

			b = AppDefaults.TextBrush;
			r.X += 1;
			r.Y += 1;
			r.Width -= 2;
			r.Height -= 2;
			g.DrawString(""+val, tc.Font, b, r);
		}
		public virtual void OnMouseDown(TableControl tc, int row, int col, MouseEventArgs e)
		{
		}
		#endregion
	}
}
