#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Drawing;
using System.Windows.Forms;

namespace ihook.PocketWidgets.Table
{
	/// <summary>
	/// Summary description for IHTableHeader.
	/// </summary>
	public abstract class TableHeader
	{
		public TableHeader()
		{
		}
		internal TableHeader(TableControl t) : this()
		{
			Table = t;
		}

		TableControl table;
		internal TableControl Table
		{
			set { table = value; }
			get { return table; }
		}

		public Size Size
		{
			get 
			{
				TableControl tc = Table;
				if(tc == null)
					return new Size(0, 0);
				return tc.Body.GetCellSize(-1, -1);
			}
		}

		public const int KnobSize = 3;
		public abstract void Paint(PaintEventArgs e);
	}
	public class TableColumnHeader : TableHeader, IColumnHeaderCellRenderer
	{
		public TableColumnHeader() : base() {}
		internal TableColumnHeader(TableControl t) : base(t) {}

		public int GetCellWidth(int col)
		{
			TableControl tc = Table;
			if(tc == null)
				return 0;
			return tc.Body.GetCellSize(-1, col).Width;
		}
		public int Height
		{
			get
			{
				TableControl tc = Table;
				if(tc == null)
					return 0;
				return tc.Body.GetCellSize(-1, -1).Height;
			}
			set 
			{
				TableControl tc = Table;
				if(tc != null)
					tc.Body.SetRowHeight(-1, value);
			}
		}

		public override void Paint(PaintEventArgs e)
		{
			TableControl tc = Table;
			if(tc == null)
				return;
			ITableModel model = tc.Model;
			if(model == null)
				return;
			TableBody body = tc.Body;

			int dx = tc.BodyOffset.X + tc.ScrollableLocation.X;
			int nc = model.NumColumns;

			int iMin, iMax;
			tc.GetBordingColumns(e.ClipRectangle, out iMin, out iMax);
			if(iMin < 0)
				iMin = 0;
			if(iMax >= nc)
				iMax = nc-1;

			Rectangle clip = e.ClipRectangle;
			Pen pLine = AppDefaults.BorderPen;
			for(int i=iMin; i<=iMax; i++) 
			{
				Point p = body.GetCellOffset(-1, i);
				Size s = body.GetCellSize(-1, i);
				Rectangle r = new Rectangle(dx+p.X+1, 0, s.Width-1, s.Height-1);
				if(tc.AllowColumnsResizing)
				{
					e.Graphics.ResetClip();
					e.Graphics.DrawLine(pLine, r.Right, r.Bottom, r.Right, r.Bottom-KnobSize);
				}
				Rectangle rClip = r;
				rClip.Intersect(clip);
				e.Graphics.Clip = new Region(rClip);
				tc.Renderers.ColumnHeader.PaintColumnHeaderCell(e.Graphics, Table, r, i, model.GetColumnTitle(i));
			}
		}
		public void PaintColumnHeaderCell(Graphics g, TableControl tc, Rectangle r, int col, string val)
		{
			Brush b = AppDefaults.TextBrush;
			g.DrawString(""+val, AppDefaults.BoldFont, b, r.X+1, r.Y+1);
		}
		public virtual void OnMouseDown(TableControl tc, int row, int col, MouseEventArgs e)
		{
		}
	}
	public class TableRowHeader : TableHeader, IRowHeaderCellRenderer
	{
		public TableRowHeader() : base() {}
		internal TableRowHeader(TableControl t) : base(t) {}

		public int GetCellHeight(int row)
		{
			TableControl tc = Table;
			if(tc == null)
				return 0;
			return tc.Body.GetCellSize(row, -1).Height;
		}
		public int Width
		{
			get
			{
				TableControl tc = Table;
				if(tc == null)
					return 0;
				return tc.Body.GetCellSize(-1, -1).Width;
			}
			set 
			{
				TableControl tc = Table;
				if(tc != null)
					tc.Body.SetColWidth(-1, value);
			}
		}

		public override void Paint(PaintEventArgs e)
		{
			TableControl tc = Table;
			if(tc == null)
				return;
			ITableModel model = tc.Model;
			if(model == null)
				return;
			TableBody body = tc.Body;

			int dy = tc.BodyOffset.Y + tc.ScrollableLocation.Y;
			int nr = model.NumRows;

			int iMin, iMax;
			tc.GetBordingRows(e.ClipRectangle, out iMin, out iMax);
			if(iMin < 0)
				iMin = 0;
			if(iMax >= nr)
				iMax = nr-1;

			Pen pLine = AppDefaults.BorderPen;
			Rectangle clip = e.ClipRectangle;
			for(int i=iMin; i<=iMax; i++)
			{
				Point p = body.GetCellOffset(i, -1);
				Size s = body.GetCellSize(i, -1);
				Rectangle r = new Rectangle(0,dy+p.Y+1, s.Width-1, s.Height-1);
				if(tc.AllowRowsResizing)
				{
					e.Graphics.ResetClip();
					e.Graphics.DrawLine(pLine, r.Right, r.Bottom, r.Right-KnobSize, r.Bottom);
				}
				Rectangle rClip = clip;
				rClip.Intersect(r);
				e.Graphics.Clip = new Region(rClip);
				tc.Renderers.RowHeader.PaintRowHeaderCell(e.Graphics, Table, r, i, model);
			}
		}
		public void PaintRowHeaderCell(Graphics g, TableControl tc, Rectangle r, int row, ITableModel model)
		{
			Brush b = AppDefaults.TextBrush;
			g.DrawString("row "+row, AppDefaults.BoldFont, b, r.X, r.Y);
		}
		public virtual void OnMouseDown(TableControl tc, int row, int col, MouseEventArgs e)
		{
		}
	}
	public class TableCorner : ICornerRenderer
	{
		public TableCorner() : base() {}

		public void PaintCorner(PaintEventArgs e, TableControl tc, Rectangle bounds)
		{
		}
	}
}
