#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Table
{
	/// <summary>
	/// this class is deliberately row oriented, because it's way TableControl
	/// are most likely to be used. if you have column edited/removed, just
	/// update all the control, sorry ;-)
	/// </summary>
	public class TableModelEvent
	{
		public enum EType
		{
			RowInserted,
			RowRemoved,
			RowUpdated,
			Structure,
			All
		}
		public TableModelEvent(ITableModel tm) : 
			this(tm, EType.All, 0, tm.NumRows)
		{
		}
		public TableModelEvent(ITableModel tm, EType aType)
			: this(tm, aType, 0, tm.NumRows)
		{
		}
		public TableModelEvent(ITableModel tm, EType aType, int row)
			: this(tm, aType, row, row)
		{
		}
		public TableModelEvent(ITableModel tm, EType aType, int rowFrom, int rowTo)
		{
			if(tm == null)
				throw new NullReferenceException("Model should not be null");
			model = tm;
			type = aType;
			rowFirst = rowFrom;
			rowLast = rowTo;
		}
		public TableModelEvent(ITableModel tm, EType aType, int rowFrom, int rowTo, int aColumn)
			: this(tm, aType, rowFrom, rowTo)
		{
			column = aColumn;
		}

		ITableModel model;
		EType type;
		int rowFirst, rowLast;
		int column = -1;

		public ITableModel Model { get { return model; } }
		public EType EventType { get { return type; } }
		public int FirstRow { get { return rowFirst; } }
		public int LastRow { get { return rowLast; } }
		/// <summary>
		/// by default it returns -1, which means all the column
		/// </summary>
		public int Column { get { return column; } }
	}
}
