#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Collections;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using ihook.PocketWidgets;
using ihook.PocketWidgets.Table.Renderers;
using ihook.PocketWidgets.Utils;

namespace ihook.PocketWidgets.Table
{
	/// <summary>
	/// manage the renderers of its table
	/// </summary>
	public class TableRendererManager
	{
		public TableRendererManager()
		{
			SetupDefaultRenderers();
		}
		protected virtual void SetupDefaultRenderers()
		{
			this[typeof(DateTime)] = new DateTimeRenderer();
			this[typeof(bool)] = new BoolRenderer();
			this[typeof(Image)] = this[typeof(Icon)] = new IconRenderer();
		}
		protected internal TableControl table;
		public TableControl Table
		{
			get { return table; }
		}

		public virtual IRowHeaderCellRenderer RowHeader
		{
			get 
			{ 
				if(rowHeaderRenderer == null) // default renderer
					return table==null ? null : table.RowHeader;
				return rowHeaderRenderer;
			}
			set { rowHeaderRenderer = value; }
		}
		IRowHeaderCellRenderer rowHeaderRenderer;
		public virtual IColumnHeaderCellRenderer ColumnHeader
		{
			get 
			{ 
				if(columnHeaderRenderer == null) // default renderer
					return table==null ? null : table.ColHeader;
				return columnHeaderRenderer;
			}
			set { columnHeaderRenderer = value; }
		}
		IColumnHeaderCellRenderer columnHeaderRenderer;

		Hashtable cellRenderer;
		public virtual ICellRenderer this[int row, int col]
		{
			get 
			{
				if(cellRenderer != null)
				{
					ulong urow = unchecked((uint) row);
					ulong ucol = unchecked((uint) col);
					ulong key = urow | (ucol << 32);
					ICellRenderer cr = (ICellRenderer) cellRenderer[key];
					if(cr != null)
						return cr;
				}
				return this[col];
			}
			set 
			{
				ulong urow = unchecked((uint) row);
				ulong ucol = unchecked((uint) col);
				ulong key = urow | (ucol << 32);
				if(value == null)
				{
					if(cellRenderer == null)
						return;
					cellRenderer.Remove(key);
				}
				else
				{
					if(cellRenderer == null)
						cellRenderer = new Hashtable();
					cellRenderer[key] = value;
				}
			}
		}

		Hashtable colRenderer;
		public virtual ICellRenderer this[int col]
		{
			get
			{
				if(colRenderer != null)
				{
					ICellRenderer cr = (ICellRenderer) colRenderer[col];
					if(cr != null)
						return cr;
				}
				ITableModel model = table.Model;
				if(model == null)
					return DefaultRender;
				return this[model.GetColumnType(col)];
			}
			set
			{
				if(value == null)
				{
					if(colRenderer == null)
						return;
					colRenderer.Remove(col);
				}
				else
				{
					if(colRenderer == null)
						colRenderer = new Hashtable();
					colRenderer[col] = value;
				}
			}
		}
		Hashtable renderers;
		public virtual ICellRenderer this[Type t]
		{
			get
			{
				if(t == null || renderers == null)
					return DefaultRender;

				ICellRenderer r = renderers[t] as ICellRenderer;
				if(r == null)
					return DefaultRender;
				return r;
			}
			set
			{
				if(t == null)
					return;
				if(renderers == null && value == null)
					return;
				if(renderers == null)
					renderers = new Hashtable();
				if(value == null)
					renderers.Remove(t);
				else
					renderers[t] = value;
			}
		}
		ICellRenderer defaultRenderer;
		public virtual ICellRenderer DefaultRender
		{
			get 
			{
				if(defaultRenderer == null)
					defaultRenderer = new TextTableRenderer();
				return defaultRenderer;
			}
			set { defaultRenderer = value; }
		}
	}
}
