//==========================================================================================
//
//		OpenNETCF.Windows.Forms.ApplicationEx
//		Copyright (c) 2003, OpenNETCF.org
//
//		This library is free software; you can redistribute it and/or modify it under 
//		the terms of the OpenNETCF.org Shared Source License.
//
//		This library is distributed in the hope that it will be useful, but 
//		WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
//		FITNESS FOR A PARTICULAR PURPOSE. See the OpenNETCF.org Shared Source License 
//		for more details.
//
//		You should have received a copy of the OpenNETCF.org Shared Source License 
//		along with this library; if not, email licensing@opennetcf.org to request a copy.
//
//		If you wish to contact the OpenNETCF Advisory Board to discuss licensing, please 
//		email licensing@opennetcf.org.
//
//		For general enquiries, email enquiries@opennetcf.org or visit our website at:
//		http://www.opennetcf.org
//
//==========================================================================================
using System;
using System.Data;
using System.Collections;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace ihook.PocketWidgets.Utils
//namespace OpenNETCF.Windows.Forms
{
	public struct MSG
	{
		public IntPtr hwnd;
		public int message;
		public IntPtr wParam;
		public IntPtr lParam;
		public int time;
		public int pt_x;
		public int pt_y;

		public Message ToMessage()
		{
			return new Message(this);
		}
	}

	public delegate void ExceptionEventHandler(Exception e);

	/// <summary>
	/// Provides static (Shared in Visual Basic) methods and properties to manage an application, such as methods to start and stop an application, to process Windows messages, and properties to get information about an application. This class cannot be inherited.
	/// </summary>
	public sealed class ApplicationEx
	{
		/// <summary>
		/// Occurs when ApplicationEx.Run exits
		/// <seealso cref="Exit"/>
		/// </summary>
		public static event EventHandler ThreadExit;
		/// <summary>
		/// Occurs when the application is about to shut down.
		/// </summary>
		public static event EventHandler ApplicationExit;
		private static ArrayList messageFilters = new ArrayList();
		private static bool messageLoop = false;
		private static Form mainForm = null;
		private static MSG msg = new MSG();
		private static bool process;

		#region --- P/Invokes ---

		[DllImport("coredll.dll", EntryPoint="PeekMessage", SetLastError=true)]
		private static extern bool PeekMessageP(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax, uint wRemoveMsg);
		[DllImport("user32.lib", EntryPoint="PeekMessage", SetLastError=true)]
		private static extern bool PeekMessageD(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax, uint wRemoveMsg);
		public static bool PeekMessage(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax, uint wRemoveMsg)
		{
			if(WinAPI.IsDevice)
				return PeekMessageP(out lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);
			else
				return PeekMessageD(out lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax, wRemoveMsg);
		}

		[DllImport("coredll.dll", EntryPoint="GetMessageW", SetLastError=true)]
		private static extern bool GetMessageP(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax);
		[DllImport("user32.dll", EntryPoint="GetMessageW", SetLastError=true)]
		private static extern bool GetMessageD(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax);
		public static bool GetMessage(out MSG lpMsg, IntPtr hWnd, uint wMsgFilterMin, uint wMsgFilterMax)
		{
			if(WinAPI.IsDevice)
				return GetMessageP(out lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);
			else
				return GetMessageD(out lpMsg, hWnd, wMsgFilterMin, wMsgFilterMax);
		}

		[DllImport("coredll.dll", EntryPoint="TranslateMessage", SetLastError=true)]
		private static extern bool TranslateMessageP(out MSG lpMsg);
		[DllImport("user32.dll", EntryPoint="TranslateMessage", SetLastError=true)]
		private static extern bool TranslateMessageD(out MSG lpMsg);
		public static bool TranslateMessage(out MSG lpMsg)
		{
			if(WinAPI.IsDevice)
				return TranslateMessageP(out lpMsg);
			else
				return TranslateMessageD(out lpMsg);
		}
		
		[DllImport("coredll.dll", EntryPoint="DispatchMessage", SetLastError=true)]
		private static extern bool DispatchMessageP(ref MSG lpMsg);
		[DllImport("user32.dll", EntryPoint="DispatchMessage", SetLastError=true)]
		private static extern bool DispatchMessageD(ref MSG lpMsg);
		public static bool DispatchMessage(ref MSG lpMsg)
		{
			if(WinAPI.IsDevice)
				return DispatchMessageP(ref lpMsg);
			else
				return DispatchMessageD(ref lpMsg);
		}
		
		[DllImport("coredll.dll", EntryPoint="PostQuitMessage", SetLastError=true)]
		private static extern void PostQuitMessageP(int nExitCode);
		[DllImport("user32.dll", EntryPoint="PostQuitMessage", SetLastError=true)]
		private static extern void PostQuitMessageD(int nExitCode);
		public static void PostQuitMessage(int nExitCode)
		{
			if(WinAPI.IsDevice)
				PostQuitMessageP(nExitCode);
			else
				PostQuitMessageD(nExitCode);
		}

		#endregion

		public static bool Pump()
		{
			return Pump(0);
		}
		public static bool Pump(uint id)
		{
			int previous = msg.message;

			if(GetMessage(out msg, IntPtr.Zero, 0, 0))
			{
				Process(ref msg);
				
				// it's probably a kill message from somewhere ...
				if(msg.message == WinAPI.WM_CLOSE && previous == 1020)
					ApplicationEx.Exit();

				return true;
			}
			else
			{
				return false;
			}
		}
		public static void Process(ref MSG msg)
		{
			process = true;

			// iterate any filters
			foreach(IMessageFilter mf in messageFilters)
			{
				Message m = Message.Create(msg.hwnd, msg.message, msg.wParam, msg.lParam);					

				// if *any* filter says not to process, we won't
				try { process = process ? !(mf.PreFilterMessage(ref m)) : false; }
				catch(Exception e) { OnException(e); }
			}

			// if we're supposed to process the message, do so
			if(process)
				try
				{
					TranslateMessage(out msg);
					DispatchMessage(ref msg);
				}
				catch(Exception e) 
				{
					// dismiss some WM_QUIT and activate event ...
					try { DoEvents(); }
					catch {}

					OnException(e); 
				}
		}

		/// <summary>
		/// call whenever an exception reach the top of the event loop uncatched
		/// </summary>
		public static event ExceptionEventHandler ExceptionEvent;
		static void OnException(Exception e)
		{
			if(ExceptionEvent != null)
				try { ExceptionEvent(e); }
				catch{}
		}

		/// <summary>
		/// remove all instance of the current event (if you use this event loop)
		/// in case only one event is enough.
		/// Use to improve the responsivity of the application (and the 
		/// user experience)
		/// </summary>
		public static void ClearCurrent()
		{
			if(clearing) // ??? apparently PeekMessage kind of dispatch the call
				return;
			clearing = true;
			try
			{
				if(messageLoop)
					while(PeekMessage(out msg, IntPtr.Zero, (uint) msg.message, (uint) msg.message, PM_REMOVE));
			}
			finally { clearing = false; }
		}
		static bool clearing = false;

		public const uint PM_REMOVE = 1;
		public const uint PM_NOREMOVE = 0;

		private static bool RunMessageLoop()
		{
			if(mainForm != null)
			{
				// if we have a form, show it
				mainForm.Visible = true;
			}

			// start up the message pump
			messageLoop = true;
			while(Pump() && messageLoop) {};
			messageLoop = false;

			// exit cleanly
			ExitThread();

			return true;
		}

		/// <summary>
		/// Gets a value indicating whether a message loop exists on this thread. 
		/// </summary>
		public static bool MessageLoop
		{
			get
			{
				return messageLoop;
			}
		}

		private static void ExitThread()
		{
			// fire the ThreadExit event if it's wired
			if(ThreadExit != null)
			{
				ThreadExit(mainForm, null);
			}

			// dispose the main form if it exists
			if(mainForm != null)
			{
				mainForm.Dispose();
			}

			if(ApplicationExit != null)
			{
				ApplicationExit(null, null);
			}

			// let the GC know it can collect
			GC.GetTotalMemory(true);
		}

		/// <summary>
		/// Begins running a standard application message loop on the current thread, without a form
		/// </summary>
		public static void Run()
		{
			RunMessageLoop();
		}

		/// <summary>
		/// Begins running a standard application message loop on the current thread, and makes the specified form visible.
		/// <seealso cref="System.Windows.Forms.Form"/>
		/// </summary>
		/// <param name="mainForm">Begins running a standard application message loop on the current thread, and makes the specified form visible.</param>
		public static void Run(Form mainForm)
		{
			mainForm.Closed += new EventHandler(MainFormExit);
			ApplicationEx.mainForm = mainForm;
			RunMessageLoop();
		}

		/// <summary>
		/// Informs all message pumps that they must terminate, and then closes all application windows after the messages have been processed.
		/// </summary>
		public static void Exit()
		{
			messageLoop = false;
			PostQuitMessage(0);
		}

		/// <summary>
		/// Processes all Windows messages currently in the message queue.
		/// </summary>
		public static void DoEvents()
		{
			DoEvents(0);
		}
		public static void DoEvents(uint id)
		{
			while(PeekMessage(out msg, IntPtr.Zero, id, id, 0))
				Pump(id);
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="toDismiss">a window to ignoreL: the parent window</param>
		public static void DoModalEvents(IntPtr toDismiss)
		{
			//ModalFilter mf = new ModalFilter(toDismiss);
			//AddMessageFilter(mf);
			try 
			{ 
				DoEvents(); 
			}
			finally 
			{ 
			//	RemoveMessageFilter(mf); 
			}
		}

		/// <summary>
		/// Adds a message filter to monitor Windows messages as they are routed to their destinations
		/// <seealso cref="IMessageFilter"/>
		/// </summary>
		/// <param name="value">The implementation of the IMessageFilter interface you want to install</param>
		public static void AddMessageFilter(IMessageFilter value)
		{
			messageFilters.Add(value);
		}

		/// <summary>
		/// Removes a message filter from the message pump of the application
		/// <seealso cref="IMessageFilter"/>
		/// </summary>
		/// <param name="value">The implementation of the IMessageFilter to remove from the application.</param>
		public static void RemoveMessageFilter(IMessageFilter value)
		{
			messageFilters.Remove(value);
		}

		/// <summary>
		/// Gets the path for the currently executing assembly file, not including the executable name.
		/// </summary>
		public static string StartupPath
		{
			get
			{
				return System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
			}
		}

		private static void MainFormExit(object sender, EventArgs e)
		{
			PostQuitMessage(0);
		}
	}
}
