/*
 * BSD Licence:
 * Copyright (c) 2001, Lloyd Dupont (ld@galador.net)
 * <ORGANIZATION> 
 * All rights reserved.
 * 
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the <ORGANIZATION> nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
using System;
using System.Collections;

namespace ihook.PocketWidgets.Utils
{
	public interface ISortable
	{
		int Count { get; }
		int CompareAt(int x1, int x2);
		void Swap(int x1, int x2);
	}

	/// <summary>an utility class to sort Array in the growing order,
	/// using the quick sort algorithm</summary>
	public class Sorter
	{
		ISortable sort;
		public Sorter()
		{
		}
		public Sorter(ISortable s) : this()
		{
			Sortable = s;
		}

		public ISortable Sortable
		{
			get { return sort; }
			set { sort = value; }
		}

		/// <summary>sort the array with QuickSort</summary>
		public void Sort() { sort1(0, sort.Count-1); }
		private void sort1(int debut, int fin)
		{
			int xm  = (debut + fin) / 2;
			int x_0 = debut;
			int x_1 = fin;
			
			while(x_0 < x_1)
			{
				while(sort.CompareAt(x_0, xm) > 0)
					x_0 ++;
				while(sort.CompareAt(xm, x_1) > 0)
					x_1 --;
				
				if(x_0 <= x_1)
				{
					if(x_0 != x_1)
					{
						sort.Swap(x_0, x_1);
						if(xm == x_0)      xm = x_1;
						else if(xm == x_1) xm = x_0;
					}
					x_0 ++;
					x_1 --;
				}
			}

			if(debut < x_1) sort1(debut, x_1);
			if(fin   > x_0) sort1(x_0, fin);
		}
	}
}
