#region -- BSD Licence --
/*
Copyright (c) 2003, 5D Services
http://www.ihook.cc
OpenSource@ihook.cc
lloyd@5dservices.com.au
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer. 

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution. 

Neither the name of "5D Services" nor the names of its contributors may be
used to endorse or promote products derived from this software without
specific prior written permission. 

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/
#endregion
using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace ihook.PocketWidgets.Utils
{
	/// <summary>
	/// export of function from the GDI api
	/// </summary>
	public class WinAPI
	{
		class IsDeviceClassInfo
		{
			public readonly bool IsDevice = Environment.OSVersion.Platform == PlatformID.WinCE;
		}
		static IsDeviceClassInfo isDevice;
		public static bool IsDevice
		{
			get
			{
				// speed trick :) (might be optional ?!?)
				if(isDevice == null)
					isDevice = new IsDeviceClassInfo();
				return isDevice.IsDevice;
			}
		}

		public const string GDI32 = "GDI32";
		public const string USER32 = "USER32";
		public const string KERNEL32 = "KERNEL32";
		public const string CORECE  = "coredll";

		#region -- GDI --
		//[Flags]
		public enum RasterOp : uint
		{
			SRCCOPY             = 0x00CC0020, /* dest = source                   */
			SRCPAINT            = 0x00EE0086, /* dest = source OR dest           */
			SRCAND              = 0x008800C6, /* dest = source AND dest          */
			SRCINVERT           = 0x00660046, /* dest = source XOR dest          */
			SRCERASE            = 0x00440328, /* dest = source AND (NOT dest )   */
			NOTSRCCOPY          = 0x00330008, /* dest = (NOT source)             */
			NOTSRCERASE         = 0x001100A6, /* dest = (NOT src) AND (NOT dest) */
			MERGECOPY           = 0x00C000CA, /* dest = (source AND pattern)     */
			MERGEPAINT          = 0x00BB0226, /* dest = (NOT source) OR dest     */
			PATCOPY             = 0x00F00021, /* dest = pattern                  */
			PATPAINT            = 0x00FB0A09, /* dest = DPSnoo                   */
			PATINVERT           = 0x005A0049, /* dest = pattern XOR dest         */
			DSTINVERT           = 0x00550009, /* dest = (NOT dest)               */
			BLACKNESS           = 0x00000042, /* dest = BLACK                    */
			WHITENESS           = 0x00FF0062 /* dest = WHITE                    */
		}

		[DllImport(GDI32, EntryPoint="BitBlt", SetLastError=true)]
		static extern bool BitBltD(
			IntPtr hdcDest, 
			int nXDest, 
			int nYDest, 
			int nWidth, 
			int nHeight, 
			IntPtr hdcSrc, 
			int nXSrc, 
			int nYSrc, 
			RasterOp dwRop
			);
		[DllImport(CORECE, EntryPoint="BitBlt", SetLastError=true)]
		static extern bool BitBltP(
			IntPtr hdcDest, 
			int nXDest, 
			int nYDest, 
			int nWidth, 
			int nHeight, 
			IntPtr hdcSrc, 
			int nXSrc, 
			int nYSrc, 
			RasterOp dwRop
			);
		public static bool BitBlt(
			IntPtr hdcDest, 
			int nXDest, 
			int nYDest, 
			int nWidth, 
			int nHeight, 
			IntPtr hdcSrc, 
			int nXSrc, 
			int nYSrc, 
			RasterOp dwRop
			)
		{
			if(IsDevice)
				return BitBltP(
					hdcDest, 
					nXDest, 
					nYDest, 
					nWidth, 
					nHeight, 
					hdcSrc, 
					nXSrc, 
					nYSrc, 
					dwRop
					);
			else
				return BitBltD(
					hdcDest, 
					nXDest, 
					nYDest, 
					nWidth, 
					nHeight, 
					hdcSrc, 
					nXSrc, 
					nYSrc, 
					dwRop
					);
		}

		[DllImport(GDI32, EntryPoint="PatBlt", SetLastError=true)]
		static extern bool PatBltD(
			IntPtr hdc, 
			int nXLeft, 
			int nYLeft, 
			int nWidth, 
			int nHeight, 
			RasterOp dwRop
			); 
		[DllImport(CORECE, EntryPoint="PatBlt", SetLastError=true)]
		static extern bool PatBltP(
			IntPtr hdc, 
			int nXLeft, 
			int nYLeft, 
			int nWidth, 
			int nHeight, 
			RasterOp dwRop
			); 
		public static bool PatBlt(
			IntPtr hdc, 
			int nXLeft, 
			int nYLeft, 
			int nWidth, 
			int nHeight, 
			RasterOp dwRop
			)
		{
			if(IsDevice)
				return PatBltP(hdc, nXLeft, nYLeft, nWidth, nHeight, dwRop);
			else
				return PatBltD(hdc, nXLeft, nYLeft, nWidth, nHeight, dwRop);
		}

		public struct COLORREF
		{
			int cValue;
			public COLORREF(int BGR)
			{
				cValue = BGR;
			}
			public COLORREF(System.Drawing.Color c)
			{
				cValue = (c.B << 16) | (c.G << 8) | c.R;
			}
			public static implicit operator COLORREF(System.Drawing.Color c)
			{
				return new COLORREF(c);
			}
		}
		[DllImport(GDI32, EntryPoint="CreateSolidBrush", SetLastError=true)]
		static extern IntPtr CreateSolidBrushD(COLORREF crColor);
		[DllImport(CORECE, EntryPoint="CreateSolidBrush", SetLastError=true)]
		static extern IntPtr CreateSolidBrushP(COLORREF crColor);
		public static IntPtr CreateSolidBrush(COLORREF crColor)
		{
			if(IsDevice)
				return CreateSolidBrushP(crColor);
			else
				return CreateSolidBrushD(crColor);
		}

		[DllImport(GDI32, EntryPoint="DeleteObject")]
		static extern bool DeleteObjectD(IntPtr obj);
		[DllImport(CORECE, EntryPoint="DeleteObject")]
		static extern bool DeleteObjectP(IntPtr obj);
		public static bool DeleteObject(IntPtr obj)
		{
			if(IsDevice)
				return DeleteObjectP(obj);
			else
				return DeleteObjectD(obj);
		}

		[DllImport(GDI32, EntryPoint="SelectObject", SetLastError=true)]
		static extern IntPtr SelectObjectD(IntPtr hdc, IntPtr hgdiobj); 
		[DllImport(CORECE, EntryPoint="SelectObject", SetLastError=true)]
		static extern IntPtr SelectObjectP(IntPtr hdc, IntPtr hgdiobj); 
		public static IntPtr SelectObject(IntPtr hdc, IntPtr hgdiobj)
		{
			if(IsDevice)
				return SelectObjectP(hdc, hgdiobj);
			else
				return SelectObjectD(hdc, hgdiobj);
		}

		[DllImport(USER32, EntryPoint="GetDC", SetLastError=true)]
		static extern IntPtr GetDCD(IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="GetDC", SetLastError=true)]
		static extern IntPtr GetDCP(IntPtr hWnd);
		public static IntPtr GetDC(IntPtr hWnd)
		{
			if(IsDevice)
				return GetDCP(hWnd);
			else
				return GetDCD(hWnd);
		}

		public static IntPtr GetDC(SmartControl c)
		{
			IntPtr h = GetDC(c.Handle);
			if(h.Equals(IntPtr.Zero))
				throw new System.ComponentModel.Win32Exception();
			return h;
		}

		[DllImport(USER32, EntryPoint="ReleaseDC")]
		static extern IntPtr ReleaseDCD(IntPtr hDC);
		[DllImport(CORECE, EntryPoint="ReleaseDC")]
		static extern IntPtr ReleaseDCP(IntPtr hDC);
		public static IntPtr ReleaseDC(IntPtr hDC)
		{
			if(IsDevice)
				return ReleaseDCP(hDC);
			else
				return ReleaseDCD(hDC);
		}

		public const uint GN_CONTEXTMENU = 1000;
		public const uint SHRG_RETURNCMD = 0x00000001;
		
		[StructLayout(LayoutKind.Sequential)]
			public class SHRGINFO
		{
			public uint cbSize = 0;
			public IntPtr hwndClient = IntPtr.Zero;
			public int x = 0; // POINT
			public int y = 0; // POINT
			public uint dwFlags = 0;
		}

		[DllImport("aygshell", EntryPoint="SHRecognizeGesture", SetLastError=true)]
		static extern uint SHRecognizeGestureP(SHRGINFO shrg);
		public static uint SHRecognizeGesture(SHRGINFO shrg)
		{
			if(IsDevice)
				return SHRecognizeGestureP(shrg);
			else // do something to return left mouse down on desktop
				return 0;
		}

		#endregion

		#region -- Window management -
		
		static System.Reflection.PropertyInfo propHandle;
		public static IntPtr GetHandle(Control c)
		{
			if(IsDevice)
			{
				bool b = c.Capture = true;
				if ( !b )
					throw new Exception("Unable to capture the mouse");
				IntPtr hWnd = GetCapture();
				if ( hWnd == IntPtr.Zero )
					throw new System.ComponentModel.Win32Exception();
				c.Capture = false;
				return hWnd;
			}
			else
			{
				if(propHandle == null)
					propHandle = typeof(Control).GetProperty("Handle");
				return (IntPtr) propHandle.GetValue(c, null);
			}
		}

		[DllImport(USER32, EntryPoint="GetFocus", SetLastError=true)]
		static extern IntPtr GetFocusD();
		[DllImport(CORECE, EntryPoint="GetFocus", SetLastError=true)]
		static extern IntPtr GetFocusP();
		public static IntPtr GetFocus()
		{
			if(IsDevice)
				return GetFocusP();
			else
				return GetFocusD();
		}

		[DllImport(USER32, EntryPoint="IsChild", SetLastError=true)]
		static extern bool IsChildD(IntPtr hWndParent, IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="IsChild", SetLastError=true)]
		static extern bool IsChildP(IntPtr hWndParent, IntPtr hWnd);
		public static bool IsChild(IntPtr hWndParent, IntPtr hWnd)
		{
			if(IsDevice)
				return IsChildP(hWndParent, hWnd);
			else
				return IsChildD(hWndParent, hWnd);
		}

		[DllImport(USER32, EntryPoint="GetParent", SetLastError=true)]
		static extern IntPtr GetParentD(IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="GetParent", SetLastError=true)]
		static extern IntPtr GetParentP(IntPtr hWnd);
		public static IntPtr GetParent(IntPtr hWnd)
		{
			if(IsDevice)
				return GetParentP(hWnd);
			else
				return GetParentD(hWnd);
		}
		public static IntPtr GetTopMostParent(IntPtr hWnd)
		{
			IntPtr p = hWnd;
			while(! p.Equals(IntPtr.Zero))
			{
				hWnd = p;
				p = GetParent(hWnd);
			}
			return hWnd;
		}

		[DllImport(USER32, EntryPoint="SetFocus", SetLastError=true)]
		static extern IntPtr SetFocusD(IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="SetFocus", SetLastError=true)]
		static extern IntPtr SetFocusP(IntPtr hWnd);
		public static IntPtr SetFocus(IntPtr hWnd)
		{
			if(IsDevice)
				return SetFocusP(hWnd);
			else
				return SetFocusD(hWnd);
		}

		[DllImport(USER32, EntryPoint="GetCapture", SetLastError=true)]
		static extern IntPtr GetCaptureD();
		[DllImport(CORECE, EntryPoint="GetCapture", SetLastError=true)]
		static extern IntPtr GetCaptureP();
		public static IntPtr GetCapture()
		{
			if(IsDevice)
				return GetCaptureP();
			else
				return GetCaptureD();
		}

		[DllImport(USER32, EntryPoint="GetWindowLong", SetLastError=true)]
		static extern int GetWindowLongD(IntPtr hWnd, GetWindowLongParam nItem);
		[DllImport(CORECE, EntryPoint="GetWindowLong", SetLastError=true)]
		static extern int GetWindowLongP(IntPtr hWnd, GetWindowLongParam nItem);
		public static int GetWindowLong(IntPtr hWnd, GetWindowLongParam nItem)
		{
			if(IsDevice)
				return GetWindowLongP(hWnd, nItem);
			else
				return GetWindowLongD(hWnd, nItem);
		}
		[DllImport(USER32, EntryPoint="SetWindowLong", SetLastError=true)]
		static extern void SetWindowLongD(IntPtr hWnd, GetWindowLongParam nItem, int nValue);
		[DllImport(CORECE, EntryPoint="SetWindowLong", SetLastError=true)]
		static extern void SetWindowLongP(IntPtr hWnd, GetWindowLongParam nItem, int nValue);
		public static void SetWindowLong(IntPtr hWnd, GetWindowLongParam nItem, int nValue)
		{
			if(IsDevice)
				SetWindowLongP(hWnd, nItem, nValue);
			else
				SetWindowLongD(hWnd, nItem, nValue);
		}
		public enum GetWindowLongParam
		{
			GWL_WNDPROC   =      (-4),
			GWL_HINSTANCE =      (-6),
			GWL_HWNDPARENT=      (-8),
			GWL_STYLE     =      (-16),
			GWL_EXSTYLE   =      (-20),
			GWL_USERDATA  =      (-21),
			GWL_ID        =      (-12),
		}

		[DllImport(USER32, EntryPoint="SetWindowPos", SetLastError=true)]
		extern static int SetWindowPosD(
			IntPtr hWnd, SetWindowPosZOrder pos,
			int X, int Y, int cx, int cy, SetWindowPosFlags uFlags);
		[DllImport(CORECE, EntryPoint="SetWindowPos", SetLastError=true)]
		extern static int SetWindowPosP(
			IntPtr hWnd, SetWindowPosZOrder pos,
			int X, int Y, int cx, int cy, SetWindowPosFlags uFlags);
		public static int SetWindowPos (
			IntPtr hWnd, SetWindowPosZOrder pos,
			int X, int Y, int cx, int cy, SetWindowPosFlags uFlags)
		{
			if(IsDevice)
				return SetWindowPosP(hWnd, pos, X, Y, cx, cy, uFlags);
			else
				return SetWindowPosD(hWnd, pos, X, Y, cx, cy, uFlags);
		}
		public enum SetWindowPosZOrder 
		{
			HWND_TOP        = 0,
			HWND_BOTTOM     = 1,
			HWND_TOPMOST    = -1,
			HWND_NOTOPMOST  = -2,
			HWND_MESSAGE = -3
		}
		public enum SetWindowPosFlags 
		{
			SWP_NOSIZE          = 0x0001,
			SWP_NOMOVE          = 0x0002,
			SWP_NOZORDER        = 0x0004,
			SWP_NOREDRAW        = 0x0008,
			SWP_NOACTIVATE      = 0x0010,
			SWP_FRAMECHANGED    = 0x0020,
			SWP_SHOWWINDOW      = 0x0040,
			SWP_HIDEWINDOW      = 0x0080,
			SWP_NOCOPYBITS      = 0x0100,
			SWP_NOOWNERZORDER   = 0x0200, 
			SWP_NOSENDCHANGING  = 0x0400,
			SWP_DRAWFRAME       = SWP_FRAMECHANGED,
			SWP_NOREPOSITION    = SWP_NOOWNERZORDER,
			SWP_DEFERERASE      = 0x2000,
			SWP_ASYNCWINDOWPOS  = 0x4000
		}
		public enum WindowStyle : int
		{
			/// <summary>
			/// Creates a window that has a thin-line border. 
			/// </summary>
			WS_BORDER = 0x00800000,
 
			/// <summary>
			/// Creates a window that has a title bar (includes the WS_BORDER style). 
			/// </summary>
			WS_CAPTION = 0x00C00000,
 
			/// <summary>
			/// Creates a child window. This style cannot be used with the WS_POPUP style. 
			/// </summary>
			WS_CHILD = 0x40000000,
 
			/// <summary>
			/// Same as the WS_CHILD style. 
			/// </summary>
			WS_CHILDWINDOW = 0x40000000,
			/// <summary>
			/// Clips child windows relative to each other; that is, when a particular child window receives a WM_PAINT message, the WS_CLIPSIBLINGS style clips all other overlapping child windows out of the region of the child window to be updated. If WS_CLIPSIBLINGS is not specified and child windows overlap, it is possible, when drawing within the client area of a child window, to draw within the client area of a neighboring child window. 
			/// </summary>
			WS_CLIPSIBLINGS = 0x04000000,
 
			/// <summary>
			/// Creates a window that is initially disabled. A disabled window cannot receive input from the user. 
			/// </summary>
			WS_DISABLED = 0x08000000,
 
			/// <summary>
			/// Creates a window that has a border of a style typically used with dialog boxes. A window with this style cannot have a title bar. 
			/// </summary>
			WS_DLGFRAME = 0x00400000,
 
			/// <summary>
			/// Specifies the first control of a group of controls. The group consists of this first control and all controls defined after it, up to the next control with the WS_GROUP style. The first control in each group usually has the WS_TABSTOP style so that the user can move from group to group. The user can subsequently change the keyboard focus from one control in the group to the next control in the group by using the direction keys. 
			/// </summary>
			WS_GROUP = 0x00020000,
 
			/// <summary>
			/// Creates a window that has a horizontal scroll bar. 
			/// </summary>
			WS_HSCROLL = 0x00100000,
 
			/// <summary>
			/// Creates a window that is initially minimized. Same as the WS_MINIMIZE style. 
			/// </summary>
			WS_ICONIC,
 
			/// <summary>
			/// Creates a window that is initially maximized. 
			/// </summary>
			WS_MAXIMIZE,
 
			/// <summary>
			/// Creates a window that has a Maximize button. Cannot be combined with the WS_EX_CONTEXTHELP style.  
			/// </summary>
			WS_MAXIMIZEBOX = 0x00020000,
 
			/// <summary>
			/// Creates a window that is initially minimized. Same as the WS_ICONIC style. 
			/// </summary>
			WS_MINIMIZE,
 
			/// <summary>
			/// Creates a window that has a Minimize button. Cannot be combined with the WS_EX_CONTEXTHELP style.  
			/// </summary>
			WS_MINIMIZEBOX = 0x00010000,
 
			/// <summary>
			/// Creates an overlapped window. An overlapped window has a title bar and a border. Same as the WS_TILED style. 
			/// </summary>
			WS_OVERLAPPED = WS_BORDER | WS_CAPTION,
 
			/// <summary>
			/// Creates an overlapped window with the WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, and WS_MAXIMIZEBOX styles. Same as the WS_TILEDWINDOW style.  
			/// </summary>
			WS_OVERLAPPEDWINDOW = WS_OVERLAPPED,
 
			/// <summary>
			/// Creates a pop-up window. This style cannot be used with the WS_CHILD style. 
			/// </summary>
			WS_POPUP = unchecked((int)0x80000000),
 
			/// <summary>
			/// Creates a pop-up window with WS_BORDER, WS_POPUP, and WS_SYSMENU styles. The WS_CAPTION and WS_POPUPWINDOW styles must be combined to make the window menu visible. 
			/// </summary>
			WS_POPUPWINDOW = WS_POPUP | WS_BORDER | WS_SYSMENU,
 
			/// <summary>
			/// Creates a window that has a sizing border. Same as the WS_THICKFRAME style. 
			/// </summary>
			WS_SIZEBOX = 0x00040000,
 
			/// <summary>
			/// Creates a window that has a Close (X) button in the non-client area. 
			/// </summary>
			WS_SYSMENU = 0x00080000,
 
			/// <summary>
			/// Specifies a control that can receive the keyboard focus when the user presses the TAB key. Pressing the TAB key changes the keyboard focus to the next control with the WS_TABSTOP style. 
			/// </summary>
			WS_TABSTOP = 0x00010000,
 
			/// <summary>
			/// Creates a window that has a sizing border. Same as the WS_SIZEBOX style. 
			/// </summary>
			WS_THICKFRAME = 0x00040000,
 
			/// <summary>
			/// Creates an overlapped window. An overlapped window has a title bar and a border. Same as the WS_OVERLAPPED style.  
			/// </summary>
			WS_TILED = WS_OVERLAPPED,
 
			/// <summary>
			/// Creates an overlapped window with the WS_OVERLAPPED, WS_CAPTION, WS_SYSMENU, WS_THICKFRAME, WS_MINIMIZEBOX, and WS_MAXIMIZEBOX styles. Same as the WS_OVERLAPPEDWINDOW style.  
			/// </summary>
			WS_TILEDWINDOW = WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU,
 
			/// <summary>
			/// Creates a window that is initially visible. 
			/// </summary>
			WS_VISIBLE = 0x10000000,
 
			/// <summary>
			/// Creates a window that has a vertical scroll bar. 
			/// </summary>
			WS_VSCROLL = 0x00200000
		}


		[DllImport(USER32, EntryPoint="GetWindowText", SetLastError=true)]
		static extern int GetWindowTextD(/*HWND*/IntPtr hWnd, /*LPTSTR*/StringBuilder lpString, int nMaxCount);
		[DllImport(CORECE, EntryPoint="GetWindowText", SetLastError=true)]
		static extern int GetWindowTextP(/*HWND*/IntPtr hWnd, /*LPTSTR*/StringBuilder lpString, int nMaxCount);
		public static int GetWindowText(/*HWND*/IntPtr hWnd, /*LPTSTR*/StringBuilder lpString, int nMaxCount)
		{
			if(IsDevice)
				return GetWindowTextP(hWnd, lpString, nMaxCount);
			else
				return GetWindowTextD(hWnd, lpString, nMaxCount);
		}

		[DllImport(USER32, EntryPoint="GetWindowTextLength", SetLastError=true)]
		static extern int GetWindowTextLengthD(/*HWND*/IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="GetWindowTextLength", SetLastError=true)]
		static extern int GetWindowTextLengthP(/*HWND*/IntPtr hWnd);
		public static int GetWindowTextLength(/*HWND*/IntPtr hWnd)
		{
			if(IsDevice)
				return GetWindowTextLengthP(hWnd);
			else
				return GetWindowTextLengthD(hWnd);
		}

		[DllImport(USER32, EntryPoint="SetWindowText", SetLastError=true)]
		static extern /*BOOL*/bool SetWindowTextD(/*HWND*/IntPtr hWnd, /*LPCTSTR*/string lpString); 
		[DllImport(CORECE, EntryPoint="SetWindowText", SetLastError=true)]
		static extern /*BOOL*/bool SetWindowTextP(/*HWND*/IntPtr hWnd, /*LPCTSTR*/string lpString); 
		public static /*BOOL*/bool SetWindowText(/*HWND*/IntPtr hWnd, /*LPCTSTR*/string lpString)
		{
			if(IsDevice)
				return SetWindowTextP(hWnd, lpString); 
			else
				return SetWindowTextD(hWnd, lpString); 
		}

		[DllImport(USER32, EntryPoint="SetForegroundWindow")]
		static extern /*BOOL*/bool SetForegroundWindowD(IntPtr/*HWND*/ hWnd); 
		[DllImport(CORECE, EntryPoint="SetForegroundWindow")]
		static extern /*BOOL*/bool SetForegroundWindowP(IntPtr/*HWND*/ hWnd); 
		public static /*BOOL*/bool SetForegroundWindow(IntPtr/*HWND*/ hWnd)
		{
			if(IsDevice)
				return SetForegroundWindowP(hWnd);
			else
				return SetForegroundWindowD(hWnd);
		}

		[DllImport(USER32, EntryPoint="GetForegroundWindow")]
		static extern /*HWND*/IntPtr GetForegroundWindowD();
		[DllImport(CORECE, EntryPoint="GetForegroundWindow")]
		static extern /*HWND*/IntPtr GetForegroundWindowP();
		public static /*HWND*/IntPtr GetForegroundWindow()
		{
			if(IsDevice)
				return GetForegroundWindowP();
			else
				return GetForegroundWindowD();
		}

		public const int SW_HIDE = 0;
		public const int SW_SHOWNORMAL = 1;
		public const int SW_NORMAL = 1;
		public const int SW_SHOWMINIMIZED = 2;
		public const int SW_MAXIMIZE = 3;
		public const int SW_SHOWMAXIMIZED = 3;
		public const int SW_SHOWNOACTIVATE = 4;
		public const int SW_SHOW = 5;
		public const int SW_MINIMIZE = 6;
		public const int SW_SHOWMINNOACTIVE = 7;
		public const int SW_SHOWNA = 8;
		public const int SW_RESTORE = 9;
		public const int SW_SHOWDEFAULT = 10;
		public const int SW_FORCEMINIMIZE = 10;
		public const int SW_MAX = 10;

		[DllImport(USER32, EntryPoint="ShowWindow")]
		public static extern /*BOOL*/bool ShowWindowD(IntPtr/*HWND*/ hWnd, int nCmdShow); 
		[DllImport(CORECE, EntryPoint="ShowWindow")]
		public static extern /*BOOL*/bool ShowWindowP(IntPtr/*HWND*/ hWnd, int nCmdShow); 
		public static /*BOOL*/bool ShowWindow(IntPtr/*HWND*/ hWnd, int nCmdShow)
		{
			if(IsDevice)
				return ShowWindowP(hWnd, nCmdShow);
			else
				return ShowWindowD(hWnd, nCmdShow);
		}

		[DllImport(USER32, EntryPoint="BringWindowToTop")]
		static extern /*BOOL*/bool BringWindowToTopD(IntPtr/*HWND*/ hWnd);
		[DllImport(CORECE, EntryPoint="BringWindowToTop")]
		static extern /*BOOL*/bool BringWindowToTopP(IntPtr/*HWND*/ hWnd);
		public static /*BOOL*/bool BringWindowToTop(IntPtr/*HWND*/ hWnd)
		{
			if(IsDevice)
				return BringWindowToTopP(hWnd);
			else
				return BringWindowToTopD(hWnd);
		}

		[DllImport(USER32, EntryPoint="MoveWindow")]
		static extern /*BOOL*/bool MoveWindowD( 
			IntPtr/*HWND*/ hWnd, 
			int X, 
			int Y, 
			int nWidth, 
			int nHeight, 
			bool/*BOOL*/ bRepaint 
			); 
		[DllImport(CORECE, EntryPoint="MoveWindow")]
		static extern /*BOOL*/bool MoveWindowP( 
			IntPtr/*HWND*/ hWnd, 
			int X, 
			int Y, 
			int nWidth, 
			int nHeight, 
			bool/*BOOL*/ bRepaint 
			); 
		public static /*BOOL*/bool MoveWindow( 
			IntPtr/*HWND*/ hWnd, 
			int X, 
			int Y, 
			int nWidth, 
			int nHeight, 
			bool/*BOOL*/ bRepaint 
			)
		{
			if(IsDevice)
				return MoveWindowP(hWnd, X, Y, nWidth, nHeight, bRepaint);
			else
				return MoveWindowD(hWnd, X, Y, nWidth, nHeight, bRepaint);
		}

		[Flags]
		public enum SWPFlags
		{
			SWP_DRAWFRAME = 32,
			SWP_FRAMECHANGED = 32,
			SWP_HIDEWINDOW = 128,
			SWP_NOACTIVATE = 16,
			SWP_NOCOPYBITS = 256,
			SWP_NOMOVE = 2,
			SWP_NOSIZE = 1,
			SWP_NOREDRAW = 8,
			SWP_NOZORDER = 4,
			SWP_SHOWWINDOW = 64,
			SWP_NOOWNERZORDER = 512,
			SWP_NOREPOSITION = 512,
			SWP_NOSENDCHANGING = 1024,
			SWP_DEFERERASE = 8192,
			SWP_ASYNCWINDOWPOS = 16384
		}

		public static readonly IntPtr HWND_BROADCAST = new IntPtr(0xFFFF);
		public static readonly IntPtr HWND_BOTTOM = new IntPtr(1);
		public static readonly IntPtr HWND_NOTOPMOST = new IntPtr(-2);
		public static readonly IntPtr HWND_TOP = new IntPtr(0);
		public static readonly IntPtr HWND_TOPMOST = new IntPtr(-1);
		public static readonly IntPtr HWND_DESKTOP = new IntPtr(0);
		public static readonly IntPtr HWND_MESSAGE = new IntPtr(-3);

		[DllImport(USER32, EntryPoint="SetWindowPos")]
		static extern /*BOOL*/ bool SetWindowPosD( 
			IntPtr/*HWND*/ hWnd, 
			IntPtr/*HWND*/ hWndInsertAfter, 
			int X, 
			int Y, 
			int cx, 
			int cy, 
			SWPFlags/*UINT*/ uFlags 
			);
		[DllImport(CORECE, EntryPoint="SetWindowPos")]
		static extern /*BOOL*/ bool SetWindowPosP( 
			IntPtr/*HWND*/ hWnd, 
			IntPtr/*HWND*/ hWndInsertAfter, 
			int X, 
			int Y, 
			int cx, 
			int cy, 
			SWPFlags/*UINT*/ uFlags 
			);
		public static /*BOOL*/ bool SetWindowPos( 
			IntPtr/*HWND*/ hWnd, 
			IntPtr/*HWND*/ hWndInsertAfter, 
			int X, 
			int Y, 
			int cx, 
			int cy, 
			SWPFlags/*UINT*/ uFlags 
			)
		{
			if(IsDevice)
				return SetWindowPosP(hWnd, hWndInsertAfter, X, Y, cx, cy, uFlags);
			else
				return SetWindowPosD(hWnd, hWndInsertAfter, X, Y, cx, cy, uFlags);
		}

		[DllImport(USER32, EntryPoint="FindWindow")]
		static extern /*HWND*/IntPtr FindWindowD(/*LPCTSTR*/string lpClassName, /*LPCTSTR*/string lpWindowName);
		[DllImport(CORECE, EntryPoint="FindWindow")]
		static extern /*HWND*/IntPtr FindWindowP(/*LPCTSTR*/string lpClassName, /*LPCTSTR*/string lpWindowName);
		public static /*HWND*/IntPtr FindWindow(/*LPCTSTR*/string lpClassName, /*LPCTSTR*/string lpWindowName)
		{
			if(IsDevice)
				return FindWindowP(lpClassName, lpWindowName);
			else
				return FindWindowD(lpClassName, lpWindowName);
		}

		[DllImport(USER32, EntryPoint="SendMessage")]
		public static extern /*LRESULT*/IntPtr SendMessageD(
			/*HWND*/IntPtr hWnd, 
			/*UINT*/int Msg, 
			/*WPARAM*/int wParam, 
			/*LPARAM*/int lParam 
			); 
		[DllImport(CORECE, EntryPoint="SendMessage")]
		public static extern /*LRESULT*/IntPtr SendMessageP(
			/*HWND*/IntPtr hWnd, 
			/*UINT*/int Msg, 
			/*WPARAM*/int wParam, 
			/*LPARAM*/int lParam 
			); 
		public static /*LRESULT*/IntPtr SendMessage(
			/*HWND*/IntPtr hWnd, 
			/*UINT*/int Msg, 
			/*WPARAM*/int wParam, 
			/*LPARAM*/int lParam 
			)
		{
			if(IsDevice)
				return SendMessageP(hWnd, Msg, wParam, lParam);
			else
				return SendMessageD(hWnd, Msg, wParam, lParam);
		}

		[DllImport(USER32, EntryPoint="IsClipboardFormatAvailable")]
		static extern bool IsClipboardFormatAvailableD(uint uFormat);
		[DllImport(CORECE, EntryPoint="IsClipboardFormatAvailable")]
		static extern bool IsClipboardFormatAvailableP(uint uFormat);
		public static bool IsClipboardFormatAvailable(uint uFormat)
		{
			if(IsDevice)
				return IsClipboardFormatAvailableP(uFormat);
			else
				return IsClipboardFormatAvailableD(uFormat);
		}

		public const int EM_CANUNDO = 198;
		public const int CF_UNICODETEXT = 13;

		public const int WM_ACTIVATE = 6;
		public const int WM_ACTIVATEAPP = 28;
		public const int WM_CUT = 768;
		public const int WM_COPY = 769;
		public const int WM_PASTE = 770;
		public const int WM_CLEAR = 771;
		public const int WM_UNDO = 772;
		public const int WM_PAINT = 15;
		public const int WM_QUIT  = 18;
		public const int WM_HELP  = 0x53;
		public const int WM_CLOSE = 16;
		public const int WM_SETTINGCHANGE = 26;
		public const int WM_LBUTTONDOWN = 513;

		public const int DOF_EXECUTABLE = 0x8001;

		public static void MakeForeground()
		{
			IntPtr hWnd = FindWindow(
				"#NETCF_AGL_PARK_",
				Assembly.GetExecutingAssembly().GetModules()[0].FullyQualifiedName);
			if ( hWnd != IntPtr.Zero )
			{
				Activate(hWnd);
				SendMessage(hWnd, WinAPI.WM_ACTIVATE, 0, 0);
				WinAPI.SendMessage(hWnd, WinAPI.DOF_EXECUTABLE, 0, 0);
				// don't work ;-(
			}
		}

		public static void Copy()
		{
			IntPtr hWnd = GetFocus();
			SendMessage(hWnd, WM_COPY, 0, 0);
		}
		public static void Cut()
		{
			IntPtr hWnd = GetFocus();
			SendMessage(hWnd, WM_CUT, 0, 0);
		}
		public static void Paste()
		{
			IntPtr hWnd = GetFocus();
			SendMessage(hWnd, WM_PASTE, 0, 0);
		}
		public static void Clear()
		{
			IntPtr hWnd = GetFocus();
			SendMessage(hWnd, WM_CLEAR, 0, 0);
		}
		public static void Undo()
		{
			IntPtr hWnd = GetFocus();
			SendMessage(hWnd, WM_UNDO, 0, 0);
		}
		public static bool CanUndo()
		{
			IntPtr hWnd = GetFocus();
			return SendMessage(hWnd, EM_CANUNDO, 0, 0) != IntPtr.Zero;
		}
		public static bool ClipboardHasText()
		{
			return IsClipboardFormatAvailable(CF_UNICODETEXT);
		}
		public static void Activate(IntPtr hWnd)
		{
			SendMessage(hWnd, DOF_EXECUTABLE, 0, 0); // no idea what it is ...
			SetFocus(hWnd);
		}

		[DllImport(USER32, EntryPoint="SetActiveWindow")]
		static extern IntPtr SetActiveWindowD(IntPtr hWnd);
		[DllImport(CORECE, EntryPoint="SetActiveWindow")]
		static extern IntPtr SetActiveWindowP(IntPtr hWnd);
		public static IntPtr SetActiveWindow(IntPtr hWnd)
		{
			if(IsDevice)
				return SetActiveWindowP(hWnd);
			else
				return SetActiveWindowD(hWnd);
		}

		#endregion

		#region -- general pupose --

		public static int GetLastError()
		{
			return Marshal.GetLastWin32Error();
		}

		[DllImport(USER32, EntryPoint="MessageBeep", SetLastError=true)]
		static extern void MessageBeepD(int Flags);
		[DllImport(CORECE, EntryPoint="MessageBeep", SetLastError=true)]
		static extern void MessageBeepP(int Flags);
		public static void MessageBeep(int Flags)
		{
			if(IsDevice)
				MessageBeepP(Flags);
			else
				MessageBeepD(Flags);
		}

		public const int MB_ICONASTERISK = 0x40;
		public const int MB_ICONEXCLAMATION = 0x30;
		public const int MB_ICONHAND = 0x10;
		public const int MB_ICONQUESTION = 0x20;
		public const int MB_OK = 0x0;

		public static void Beep()
		{
			MessageBeep(MB_OK);
		}
		public enum BeepSound
		{
			Asterisk = MB_ICONASTERISK,
			Exclamation = MB_ICONEXCLAMATION, 
			Hand = MB_ICONHAND,
			Question = MB_ICONQUESTION,
			Ok = MB_OK
		}
		public static void Beep(BeepSound bs)
		{
			MessageBeep((int) bs);
		}

		#endregion

		#region CreateProcess & LastError PInvoke

		public class PInfo
		{
			public IntPtr hProcess; 
			public IntPtr hThread; 
			public uint dwProcessId; 
			public uint dwThreadId; 
		}

		[DllImport(KERNEL32, EntryPoint="TerminateProcess", SetLastError=true)]
		public static extern int TerminateProcessD(
			IntPtr hProcess, 
			uint uExitCode
			);
		[DllImport(CORECE, EntryPoint="TerminateProcess", SetLastError=true)]
		public static extern int TerminateProcessP(
			IntPtr hProcess, 
			uint uExitCode
			);
		public static int TerminateProcess(
			IntPtr hProcess, 
			uint uExitCode
			)
		{
			if(IsDevice)
				return TerminateProcessP(hProcess, uExitCode);
			else
				return TerminateProcessD(hProcess, uExitCode);
		}

		[DllImport(KERNEL32, EntryPoint="CreateProcess", SetLastError=true)]
		extern static int CreateProcessD( 
			string imageName,
			string cmdLine,
			IntPtr lpProcessAttributes,
			IntPtr lpThreadAttributes,
			int boolInheritHandles,
			int dwCreationFlags,
			IntPtr lpEnvironment,
			IntPtr lpszCurrentDir,
			byte [] si,
			PInfo pi );
		[DllImport(CORECE, EntryPoint="CreateProcess", SetLastError=true)]
		extern static int CreateProcessP( 
			string imageName,
			string cmdLine,
			IntPtr lpProcessAttributes,
			IntPtr lpThreadAttributes,
			int boolInheritHandles,
			int dwCreationFlags,
			IntPtr lpEnvironment,
			IntPtr lpszCurrentDir,
			byte [] si,
			PInfo pi );
		public static int CreateProcess( 
			string imageName,
			string cmdLine,
			IntPtr lpProcessAttributes,
			IntPtr lpThreadAttributes,
			int boolInheritHandles,
			int dwCreationFlags,
			IntPtr lpEnvironment,
			IntPtr lpszCurrentDir,
			byte [] si,
			PInfo pi )
		{
			if(IsDevice)
				return CreateProcessP(
					imageName,
					cmdLine,
					lpProcessAttributes,
					lpThreadAttributes,
					boolInheritHandles,
					dwCreationFlags,
					lpEnvironment,
					lpszCurrentDir,
					si,
					pi );
			else
				return CreateProcessD(
					imageName,
					cmdLine,
					lpProcessAttributes,
					lpThreadAttributes,
					boolInheritHandles,
					dwCreationFlags,
					lpEnvironment,
					lpszCurrentDir,
					si,
					pi );
		}

		public static bool CreateProcess( string ExeName, string CmdLine, out IntPtr hProcess)
		{
			PInfo pi = new PInfo();
			int ret = CreateProcess(
				ExeName,
				CmdLine,
				IntPtr.Zero,
				IntPtr.Zero,
				0,
				0,
				IntPtr.Zero,
				IntPtr.Zero,
				null,
				pi);
			hProcess = pi.hProcess;
			return ret != 0;
		}
		#endregion
	}
}
