using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.Data;

using ihook.PocketWidgets;

namespace ihookDB.Widgets.Paint
{
	/// <summary>
	/// Summary description for ColorChooser.
	/// </summary>
	public class ColorChooser : Control
	{
		public ColorChooser()
		{
			Size = new Size(59, 124);
		}

		public static readonly Color[] COLORS = new Color[]
			{
				Color.FromArgb(unchecked((int) 0xFF000000)),
				Color.FromArgb(unchecked((int) 0xFFFFFFFF)),
				Color.FromArgb(unchecked((int) 0xFF800000)),
				Color.FromArgb(unchecked((int) 0xFFFF0000)),
				Color.FromArgb(unchecked((int) 0xFF008000)),
				Color.FromArgb(unchecked((int) 0xFF808000)),
				Color.FromArgb(unchecked((int) 0xFFFF8000)),
				Color.FromArgb(unchecked((int) 0xFF00FF00)),
				Color.FromArgb(unchecked((int) 0xFF80FF00)),
				Color.FromArgb(unchecked((int) 0xFFFFFF00)),
				Color.FromArgb(unchecked((int) 0xFF000080)),
				Color.FromArgb(unchecked((int) 0xFF800080)),
				Color.FromArgb(unchecked((int) 0xFFFF0080)),
				Color.FromArgb(unchecked((int) 0xFF008080)),
				Color.FromArgb(unchecked((int) 0xFF808080)),
				Color.FromArgb(unchecked((int) 0xFFFF8080)),
				Color.FromArgb(unchecked((int) 0xFF00FF80)),
				Color.FromArgb(unchecked((int) 0xFF80FF80)),
				Color.FromArgb(unchecked((int) 0xFFFFFF80)),
				Color.FromArgb(unchecked((int) 0xFF0000FF)),
				Color.FromArgb(unchecked((int) 0xFF8000FF)),
				Color.FromArgb(unchecked((int) 0xFFFF00FF)),
				Color.FromArgb(unchecked((int) 0xFF0080FF)),
				Color.FromArgb(unchecked((int) 0xFF8080FF)),
				Color.FromArgb(unchecked((int) 0xFFFF80FF)),
				Color.FromArgb(unchecked((int) 0xFF00FFFF)),
				Color.FromArgb(unchecked((int) 0xFF80FFFF))
			};
		int selected = 0;
		public Color SelectedColor
		{
			get { return COLORS[selected]; }
		}
		public int SelectedIndex 
		{ 
			get { return selected; }
			set
			{
				if(selected == value)
					return;
				selected = value;
				if(selected < 0 || selected >= COLORS.Length)
					selected = 0;
				Invalidate(GetBounds(0));
				if(ColorChanged != null)
					ColorChanged(this, EventArgs.Empty);
			}
		}

		protected override void OnPaint(PaintEventArgs e)
		{
			base.OnPaint (e);
			Pen rp = new Pen(Color.Red);
			Pen bp = new Pen(Color.Black);
			for(int i=0; i<=COLORS.Length; i++)
			{
				Rectangle r = GetBounds(i);

				Color c = i==0 ? SelectedColor : COLORS[i-1];
				Brush b = new SolidBrush(c);
				e.Graphics.FillRectangle(b, r);
				b.Dispose();
				e.Graphics.DrawRectangle(i==0 ? rp : bp, r);
			}
			rp.Dispose();
			bp.Dispose();
		}

		protected override void OnMouseDown(MouseEventArgs e)
		{
			base.OnMouseDown (e);
			int x = e.X;
			int y = e.Y;
			for(int i=1; i<= COLORS.Length; i++)
				if(GetBounds(i).Contains(x, y))
				{
					SelectedIndex = i-1;
					return;
				}
		}

		const int NCOL = 4;

		/// <summary>
		/// get the color rectangle coordinate. 0 is the selected
		/// color and the other are sample.
		/// </summary>
		/// <param name="square">the square number</param>
		/// <returns>the position</returns>
		Rectangle GetBounds(int square)
		{
			if(square == 0)
				return new Rectangle(0, 0, 18, 18);
			int c = (square-1)/NCOL;
			int r = (square-1)%NCOL;
			return new Rectangle(r*15, c*15+20, 13, 13);
		}

		public event EventHandler ColorChanged;
	}
}
